@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 2016-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2016-2022 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -27,10 +27,10 @@
 #include <string.h>
 #include <wget.h>
 
-int main(int argc G_GNUC_WGET_UNUSED, const char *const *argv G_GNUC_WGET_UNUSED)
+int main(int argc WGET_GCC_UNUSED, const char *const *argv WGET_GCC_UNUSED)
 {
-	wget_http_connection_t *conn = NULL;
-	wget_http_response_t *resp;
+	wget_http_connection *conn = NULL;
+	wget_http_response *resp;
 
 	// set up libwget global configuration
 	wget_global_init(
@@ -42,8 +42,8 @@ int main(int argc G_GNUC_WGET_UNUSED, const char *const *argv G_GNUC_WGET_UNUSED
 	// This is the text that we want to convert into a GFX
 	const char *text = "alice->bob: authentication request\nbob-->alice: response";
 	const char *style = "qsd";
-	wget_buffer_t *url = wget_buffer_alloc(128);
-	wget_buffer_t *body = wget_buffer_alloc(128);
+	wget_buffer *url = wget_buffer_alloc(128);
+	wget_buffer *body = wget_buffer_alloc(128);
 
 
 	wget_buffer_strcpy(body, "message=");
