@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 2017-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2017-2022 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -33,7 +33,7 @@
 
 int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size)
 {
-	wget_robots_t *robots;
+	wget_robots *robots;
 	char *in = (char *) malloc(size + 1);
 
 	assert(in != NULL);
@@ -42,8 +42,8 @@ int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size)
 	memcpy(in, data, size);
 	in[size] = 0;
 
-	robots = wget_robots_parse(in, "wget2");
-	wget_robots_free(&robots);
+	if (wget_robots_parse(&robots, in, "wget2") == WGET_E_SUCCESS)
+		wget_robots_free(&robots);
 
 	free(in);
 
