@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2012-2015 Tim Ruehsen
- * Copyright(c) 2015-2019 Free Software Foundation, Inc.
+ * Copyright (c) 2012-2015 Tim Ruehsen
+ * Copyright (c) 2015-2022 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -28,15 +28,14 @@
 #ifndef WGET_WGET_H
 #define WGET_WGET_H
 
-#include <stddef.h>
+#include <stddef.h> // size_t
 #include <sys/types.h>
 #include <unistd.h>
-#include <stdarg.h>
-#include <stdio.h>
+#include <stdarg.h> // va_list
+#include <stdio.h> // FILE
 #include <stdlib.h>
-#include <time.h>
-#include <stdbool.h>
-#include <inttypes.h>
+#include <stdbool.h> // bool
+#include <stdint.h> // int64_t
 
 #ifdef WGETVER_FILE
 #   include WGETVER_FILE
@@ -185,7 +184,7 @@
 #	define WGET_END_DECLS
 #endif
 
-/// define MALLOC_RETURNS_NONNULL when using appropriate implementations of the alloc functions
+// define MALLOC_RETURNS_NONNULL when using appropriate implementations of the alloc functions
 #ifdef MALLOC_RETURNS_NONNULL
 #  define RETURNS_NONNULL WGET_GCC_RETURNS_NONNULL
 #  define NULLABLE
@@ -228,27 +227,29 @@ WGET_BEGIN_DECLS
 #define WGET_NET_FAMILY_EXCLUSIVE 1016
 #define WGET_NET_FAMILY_PREFERRED 1017
 #define WGET_TCP_FASTFORWARD  1018
+#define WGET_BIND_INTERFACE  1019
 
-#define WGET_HTTP_URL                 2000
-#define WGET_HTTP_URL_ENCODING        2001
-#define WGET_HTTP_URI                 2002
-#define WGET_HTTP_COOKIE_STORE        2003
-#define WGET_HTTP_HEADER_ADD          2004
-//#define WGET_HTTP_HEADER_DEL        2005
-//#define WGET_HTTP_HEADER_SET        2006
-//#define WGET_HTTP_BIND_ADDRESS      2007
-#define WGET_HTTP_CONNECTION_PTR      2008
-#define WGET_HTTP_RESPONSE_KEEPHEADER 2009
-#define WGET_HTTP_MAX_REDIRECTIONS    2010
-#define WGET_HTTP_BODY_SAVEAS_STREAM  2011
-#define WGET_HTTP_BODY_SAVEAS_FILE    2012
-#define WGET_HTTP_BODY_SAVEAS_FD      2013
-#define WGET_HTTP_BODY_SAVEAS_FUNC    2014
-#define WGET_HTTP_HEADER_FUNC         2015
-#define WGET_HTTP_SCHEME              2016
-#define WGET_HTTP_BODY                2017
-#define WGET_HTTP_BODY_SAVEAS         2018
-#define WGET_HTTP_USER_DATA           2019
+#define WGET_HTTP_URL                   2000
+#define WGET_HTTP_URL_ENCODING          2001
+#define WGET_HTTP_URI                   2002
+#define WGET_HTTP_COOKIE_STORE          2003
+#define WGET_HTTP_HEADER_ADD            2004
+//#define WGET_HTTP_HEADER_DEL          2005
+//#define WGET_HTTP_HEADER_SET          2006
+//#define WGET_HTTP_BIND_ADDRESS        2007
+#define WGET_HTTP_CONNECTION_PTR        2008
+#define WGET_HTTP_RESPONSE_KEEPHEADER   2009
+#define WGET_HTTP_MAX_REDIRECTIONS      2010
+#define WGET_HTTP_BODY_SAVEAS_STREAM    2011
+#define WGET_HTTP_BODY_SAVEAS_FILE      2012
+#define WGET_HTTP_BODY_SAVEAS_FD        2013
+#define WGET_HTTP_BODY_SAVEAS_FUNC      2014
+#define WGET_HTTP_HEADER_FUNC           2015
+#define WGET_HTTP_SCHEME                2016
+#define WGET_HTTP_BODY                  2017
+#define WGET_HTTP_BODY_SAVEAS           2018
+#define WGET_HTTP_USER_DATA             2019
+#define WGET_HTTP_RESPONSE_IGNORELENGTH 2020
 
 // definition of error conditions
 typedef enum {
@@ -261,13 +262,10 @@ typedef enum {
 	WGET_E_HANDSHAKE = -6, /* general TLS handshake failure */
 	WGET_E_CERTIFICATE = -7, /* general TLS certificate failure */
 	WGET_E_TLS_DISABLED = -8, /* TLS was not enabled at compile time */
-	WGET_E_GPG_DISABLED = -9, /* GPGME was not enabled at compile time */
-	WGET_E_GPG_VER_FAIL = -10, /* 1 or more non-valid signatures */
-	WGET_E_GPG_VER_ERR = -11, /* Verification failed, GPGME error */
-	WGET_E_XML_PARSE_ERR = -12, /* XML parsing failed */
-	WGET_E_OPEN = -13, /* Failed to open file */
-	WGET_E_IO = -14, /* General I/O error (read/write/stat/...) */
-	WGET_E_UNSUPPORTED = -15, /* Unsupported function */
+	WGET_E_XML_PARSE_ERR = -9, /* XML parsing failed */
+	WGET_E_OPEN = -10, /* Failed to open file */
+	WGET_E_IO = -11, /* General I/O error (read/write/stat/...) */
+	WGET_E_UNSUPPORTED = -12, /* Unsupported function */
 } wget_error;
 
 WGETAPI const char *
@@ -333,7 +331,7 @@ WGETAPI int
 WGETAPI int
 	wget_strncasecmp(const char *s1, const char *s2, size_t n) WGET_GCC_PURE;
 WGETAPI void
-	wget_memtohex(const unsigned char *src, size_t src_len, char *dst, size_t dst_size);
+	wget_memtohex(const unsigned char * restrict src, size_t src_len, char * restrict dst, size_t dst_size);
 WGETAPI void
 	wget_millisleep(int ms);
 WGETAPI long long
@@ -355,11 +353,11 @@ WGETAPI ssize_t
 WGETAPI ssize_t
 	wget_getline(char **buf, size_t *bufsize, FILE *fp);
 WGETAPI FILE * NULLABLE
-	wget_vpopenf(const char *type, const char *fmt, va_list args) WGET_GCC_PRINTF_FORMAT(2,0);
+	wget_vpopenf(const char *type, const char *restrict fmt, va_list args) WGET_GCC_PRINTF_FORMAT(2,0);
 WGETAPI FILE * NULLABLE
-	wget_popenf(const char *type, const char *fmt, ...) WGET_GCC_PRINTF_FORMAT(2,3);
+	wget_popenf(const char *type, const char *restrict fmt, ...) WGET_GCC_PRINTF_FORMAT(2,3);
 WGETAPI FILE * NULLABLE
-	wget_popen2f(FILE **fpin, FILE **fpout, const char *fmt, ...) WGET_GCC_PRINTF_FORMAT(3,4);
+	wget_popen2f(FILE **fpin, FILE **fpout, const char *restrict fmt, ...) WGET_GCC_PRINTF_FORMAT(3,4);
 WGETAPI pid_t
 	wget_fd_popen3(int *fdin, int *fdout, int *fderr, const char *const *argv);
 WGETAPI pid_t
@@ -376,7 +374,7 @@ WGETAPI int
 	wget_memiconv(const char *src_encoding, const void *src, size_t srclen, const char *dst_encoding, char **out, size_t *outlen);
 WGETAPI char * NULLABLE
 	wget_striconv(const char *src, const char *src_encoding, const char *dst_encoding) WGET_GCC_MALLOC;
-WGETAPI int
+WGETAPI bool
 	wget_str_needs_encoding(const char *s) WGET_GCC_PURE;
 WGETAPI bool
 	wget_str_is_valid_utf8(const char *utf8) WGET_GCC_PURE;
@@ -488,8 +486,12 @@ LIBWGET_WARN_UNUSED_RESULT WGET_GCC_ALLOC_SIZE(2)
 WGETAPI char * NULLABLE
 	wget_strmemdup(const void *m, size_t n);
 
-WGETAPI void
-	wget_strmemcpy(char *s, size_t ssize, const void *m, size_t n);
+WGETAPI size_t
+	wget_strmemcpy(char *restrict s, size_t ssize, const void *restrict m, size_t n);
+
+LIBWGET_WARN_UNUSED_RESULT WGET_GCC_NONNULL_ALL
+WGETAPI void * NULLABLE
+	wget_strmemcpy_a(char *s, size_t ssize, const void *m, size_t n);
 
 /*
  * Base64 routines
@@ -508,9 +510,9 @@ WGETAPI size_t
 WGETAPI size_t
 	wget_base64_urlencode(char *restrict dst, const char *restrict src, size_t n) WGET_GCC_NONNULL_ALL;
 WGETAPI char * NULLABLE
-	wget_base64_decode_alloc(const char *restrict src, size_t n, size_t *outlen) WGET_GCC_NONNULL((1));
+	wget_base64_decode_alloc(const char *src, size_t n, size_t *outlen) WGET_GCC_NONNULL((1));
 WGETAPI char * NULLABLE
-	wget_base64_encode_alloc(const char *restrict src, size_t n) WGET_GCC_NONNULL_ALL;
+	wget_base64_encode_alloc(const char *src, size_t n) WGET_GCC_NONNULL_ALL;
 WGETAPI char * NULLABLE
 	wget_base64_encode_vprintf_alloc(const char *restrict fmt, va_list args) WGET_GCC_PRINTF_FORMAT(1,0) WGET_GCC_NONNULL_ALL;
 WGETAPI char * NULLABLE
@@ -583,36 +585,36 @@ WGETAPI size_t
 WGETAPI char *
 	wget_buffer_trim(wget_buffer *buf);
 WGETAPI size_t
-	wget_buffer_vprintf_append(wget_buffer *buf, const char *fmt, va_list args) WGET_GCC_PRINTF_FORMAT(2,0);
+	wget_buffer_vprintf_append(wget_buffer *buf, const char *restrict fmt, va_list args) WGET_GCC_PRINTF_FORMAT(2,0);
 WGETAPI size_t
-	wget_buffer_printf_append(wget_buffer *buf, const char *fmt, ...) WGET_GCC_PRINTF_FORMAT(2,3);
+	wget_buffer_printf_append(wget_buffer *buf, const char *restrict fmt, ...) WGET_GCC_PRINTF_FORMAT(2,3);
 WGETAPI size_t
-	wget_buffer_vprintf(wget_buffer *buf, const char *fmt, va_list args) WGET_GCC_PRINTF_FORMAT(2,0);
+	wget_buffer_vprintf(wget_buffer *buf, const char *restrict fmt, va_list args) WGET_GCC_PRINTF_FORMAT(2,0);
 WGETAPI size_t
-	wget_buffer_printf(wget_buffer *buf, const char *fmt, ...) WGET_GCC_PRINTF_FORMAT(2,3);
+	wget_buffer_printf(wget_buffer *buf, const char *restrict fmt, ...) WGET_GCC_PRINTF_FORMAT(2,3);
 
 /*
  * Printf-style routines
  */
 
 WGETAPI size_t
-	wget_vasprintf(char **strp, const char *fmt, va_list args) WGET_GCC_PRINTF_FORMAT(2,0);
+	wget_vasprintf(char **restrict strp, const char *restrict fmt, va_list args) WGET_GCC_PRINTF_FORMAT(2,0);
 WGETAPI size_t
-	wget_asprintf(char **strp, const char *fmt, ...) WGET_GCC_PRINTF_FORMAT(2,3);
+	wget_asprintf(char **restrict strp, const char *restrict fmt, ...) WGET_GCC_PRINTF_FORMAT(2,3);
 WGETAPI char * NULLABLE
-	wget_vaprintf(const char *fmt, va_list args) WGET_GCC_PRINTF_FORMAT(1,0);
+	wget_vaprintf(const char *restrict fmt, va_list args) WGET_GCC_PRINTF_FORMAT(1,0);
 WGETAPI char * NULLABLE
-	wget_aprintf(const char *fmt, ...) WGET_GCC_PRINTF_FORMAT(1,2);
+	wget_aprintf(const char *restrict fmt, ...) WGET_GCC_PRINTF_FORMAT(1,2);
 WGETAPI size_t
-	wget_vfprintf(FILE *fp, const char *fmt, va_list args) WGET_GCC_PRINTF_FORMAT(2,0);
+	wget_vfprintf(FILE *restrict fp, const char *restrict fmt, va_list args) WGET_GCC_PRINTF_FORMAT(2,0);
 WGETAPI size_t
-	wget_fprintf(FILE *fp, const char *fmt, ...) WGET_GCC_PRINTF_FORMAT(2,3);
+	wget_fprintf(FILE *restrict fp, const char *restrict fmt, ...) WGET_GCC_PRINTF_FORMAT(2,3);
 WGETAPI size_t
-	wget_printf(const char *fmt, ...) WGET_GCC_PRINTF_FORMAT(1,2);
+	wget_printf(const char *restrict fmt, ...) WGET_GCC_PRINTF_FORMAT(1,2);
 WGETAPI size_t
-	wget_vsnprintf(char *str, size_t size, const char *fmt, va_list args) WGET_GCC_PRINTF_FORMAT(3,0);
+	wget_vsnprintf(char *restrict str, size_t size, const char *restrict fmt, va_list args) WGET_GCC_PRINTF_FORMAT(3,0);
 WGETAPI size_t
-	wget_snprintf(char *str, size_t size, const char *fmt, ...) WGET_GCC_PRINTF_FORMAT(3,4);
+	wget_snprintf(char *restrict str, size_t size, const char *restrict fmt, ...) WGET_GCC_PRINTF_FORMAT(3,4);
 
 /*
  * Logger routines
@@ -645,19 +647,19 @@ WGETAPI bool
 #define WGET_LOGGER_DEBUG  3
 
 WGETAPI void
-	wget_info_vprintf(const char *fmt, va_list args) WGET_GCC_NONNULL_ALL WGET_GCC_PRINTF_FORMAT(1,0);
+	wget_info_vprintf(const char *restrict fmt, va_list args) WGET_GCC_NONNULL_ALL WGET_GCC_PRINTF_FORMAT(1,0);
 WGETAPI void
-	wget_info_printf(const char *fmt, ...) WGET_GCC_NONNULL((1)) WGET_GCC_PRINTF_FORMAT(1,2);
+	wget_info_printf(const char *restrict fmt, ...) WGET_GCC_NONNULL((1)) WGET_GCC_PRINTF_FORMAT(1,2);
 WGETAPI void
-	wget_error_vprintf(const char *fmt, va_list args) WGET_GCC_NONNULL_ALL WGET_GCC_PRINTF_FORMAT(1,0);
+	wget_error_vprintf(const char *restrict fmt, va_list args) WGET_GCC_NONNULL_ALL WGET_GCC_PRINTF_FORMAT(1,0);
 WGETAPI void
-	wget_error_printf(const char *fmt, ...) WGET_GCC_NONNULL((1)) WGET_GCC_PRINTF_FORMAT(1,2);
+	wget_error_printf(const char *restrict fmt, ...) WGET_GCC_NONNULL((1)) WGET_GCC_PRINTF_FORMAT(1,2);
 WGETAPI void WGET_GCC_NONNULL((1)) WGET_GCC_NORETURN WGET_GCC_PRINTF_FORMAT(1,2)
-	wget_error_printf_exit(const char *fmt, ...);
+	wget_error_printf_exit(const char *restrict fmt, ...);
 WGETAPI void
-	wget_debug_vprintf(const char *fmt, va_list args) WGET_GCC_NONNULL_ALL WGET_GCC_PRINTF_FORMAT(1,0);
+	wget_debug_vprintf(const char *restrict fmt, va_list args) WGET_GCC_NONNULL_ALL WGET_GCC_PRINTF_FORMAT(1,0);
 WGETAPI void
-	wget_debug_printf(const char *fmt, ...) WGET_GCC_NONNULL((1)) WGET_GCC_PRINTF_FORMAT(1,2);
+	wget_debug_printf(const char *restrict fmt, ...) WGET_GCC_NONNULL((1)) WGET_GCC_PRINTF_FORMAT(1,2);
 WGETAPI void
 	wget_debug_write(const char *buf, size_t len) WGET_GCC_NONNULL_ALL;
 WGETAPI wget_logger *
@@ -692,9 +694,9 @@ WGETAPI int
 WGETAPI int
 	wget_vector_add(wget_vector *v, const void *elem) WGET_GCC_NONNULL((2));
 WGETAPI int
-	wget_vector_add_vprintf(wget_vector *v, const char *fmt, va_list args) WGET_GCC_PRINTF_FORMAT(2,0);
+	wget_vector_add_vprintf(wget_vector *v, const char *restrict fmt, va_list args) WGET_GCC_PRINTF_FORMAT(2,0);
 WGETAPI int
-	wget_vector_add_printf(wget_vector *v, const char *fmt, ...) WGET_GCC_PRINTF_FORMAT(2,3);
+	wget_vector_add_printf(wget_vector *v, const char *restrict fmt, ...) WGET_GCC_PRINTF_FORMAT(2,3);
 WGETAPI int
 	wget_vector_replace(wget_vector *v, const void *elem, int pos) WGET_GCC_NONNULL((2));
 WGETAPI int
@@ -764,7 +766,7 @@ WGETAPI void
 WGETAPI void
 	wget_hashmap_clear(wget_hashmap *h);
 WGETAPI int
-	wget_hashmap_get(const wget_hashmap *h, const void *key, void **value);
+	wget_hashmap_get(const wget_hashmap *h, const void *key, void **value) WGET_GCC_UNUSED_RESULT;
 #define wget_hashmap_get(a, b, c) wget_hashmap_get((a), (b), (void **)(c))
 WGETAPI int
 	wget_hashmap_contains(const wget_hashmap *h, const void *key);
@@ -870,7 +872,7 @@ int wget_stringmap_put(wget_stringmap *h, const char *key, const void *value)
  *
  * Neither \p h nor \p key must be %NULL.
  */
-static inline
+static inline WGET_GCC_UNUSED_RESULT
 int wget_stringmap_get(const wget_stringmap *h, const char *key, void **value)
 {
 	return wget_hashmap_get(h, key, value);
@@ -1082,7 +1084,7 @@ void wget_stringmap_set_resize_factor(wget_stringmap *h, float factor)
  * When iterating over a stringmap, the order of returned key/value pairs is not defined.
  */
 static inline
-void * NULLABLE wget_stringmap_iterator_next(wget_stringmap_iterator *h, char **value)
+void * NULLABLE wget_stringmap_iterator_next(wget_stringmap_iterator *h, void **value)
 {
 	return wget_hashmap_iterator_next(h, (void **) value);
 }
@@ -1145,7 +1147,8 @@ typedef enum {
 	wget_content_encoding_bzip2 = 5,
 	wget_content_encoding_brotli = 6,
 	wget_content_encoding_zstd = 7,
-	wget_content_encoding_max = 8
+	wget_content_encoding_lzip = 8,
+	wget_content_encoding_max = 9
 } wget_content_encoding;
 
 WGETAPI WGET_GCC_PURE wget_content_encoding
@@ -1157,7 +1160,7 @@ WGETAPI wget_decompressor * NULLABLE
 WGETAPI void
 	wget_decompress_close(wget_decompressor *dc);
 WGETAPI int
-	wget_decompress(wget_decompressor *dc, char *src, size_t srclen);
+	wget_decompress(wget_decompressor *dc, const char *src, size_t srclen);
 WGETAPI void
 	wget_decompress_set_error_handler(wget_decompressor *dc, wget_decompressor_error_handler *error_handler);
 WGETAPI void * NULLABLE
@@ -1172,6 +1175,9 @@ typedef enum {
 	WGET_IRI_SCHEME_HTTPS = 1
 } wget_iri_scheme;
 
+// flags for wget_iri_get_basename()
+#define WGET_IRI_WITH_QUERY  1
+
 /**
  * \ingroup libwget-iri
  *
@@ -1292,8 +1298,6 @@ WGETAPI bool
 	wget_iri_isreserved(char c) WGET_GCC_CONST;
 WGETAPI bool
 	wget_iri_isunreserved(char c) WGET_GCC_CONST;
-WGETAPI bool
-	wget_iri_isunreserved_path(char c) WGET_GCC_CONST;
 WGETAPI int
 	wget_iri_compare(wget_iri *iri1, wget_iri *iri2) WGET_GCC_PURE;
 WGETAPI char *
@@ -1303,13 +1307,13 @@ WGETAPI char *
 WGETAPI wget_iri *
 	wget_iri_parse(const char *uri, const char *encoding);
 WGETAPI wget_iri * NULLABLE
-	wget_iri_parse_base(wget_iri *base, const char *url, const char *encoding);
+	wget_iri_parse_base(const wget_iri *base, const char *url, const char *encoding);
 WGETAPI wget_iri * NULLABLE
 	wget_iri_clone(const wget_iri *iri);
 WGETAPI const char * NULLABLE
-	wget_iri_get_connection_part(wget_iri *iri);
+	wget_iri_get_connection_part(const wget_iri *iri, wget_buffer *buf);
 WGETAPI const char *
-	wget_iri_relative_to_abs(wget_iri *base, const char *val, size_t len, wget_buffer *buf);
+	wget_iri_relative_to_abs(const wget_iri *base, const char *val, size_t len, wget_buffer *buf);
 WGETAPI const char *
 	wget_iri_escape(const char *src, wget_buffer *buf);
 WGETAPI const char *
@@ -1325,7 +1329,7 @@ WGETAPI char *
 WGETAPI char *
 	wget_iri_get_query_as_filename(const wget_iri *iri, wget_buffer *buf, const char *encoding) WGET_GCC_NONNULL((1,2));
 WGETAPI char *
-	wget_iri_get_filename(const wget_iri *iri, wget_buffer *buf, const char *encoding) WGET_GCC_NONNULL((1,2));
+	wget_iri_get_basename(const wget_iri *iri, wget_buffer *buf, const char *encoding, int flags) WGET_GCC_NONNULL((1,2));
 WGETAPI wget_iri_scheme
 	wget_iri_set_scheme(wget_iri *iri, wget_iri_scheme scheme);
 WGETAPI const char * NULLABLE
@@ -1368,7 +1372,7 @@ WGETAPI void
 WGETAPI void
 	wget_cookie_db_free(wget_cookie_db **cookie_db);
 WGETAPI void
-	wget_cookie_set_keep_session_cookies(wget_cookie_db *cookie_db, int keep);
+	wget_cookie_set_keep_session_cookies(wget_cookie_db *cookie_db, bool keep);
 WGETAPI int
 	wget_cookie_db_save(wget_cookie_db *cookie_db, const char *fname);
 WGETAPI int
@@ -1396,11 +1400,11 @@ typedef struct wget_hsts_db_st wget_hsts_db;
  * See tests/test-plugin-dummy.c and tests/Makefile.am for details.
  */
 
-typedef int wget_hsts_host_match_fn(const wget_hsts_db *, const char *hsts_db, uint16_t port);
+typedef int wget_hsts_host_match_fn(const wget_hsts_db *hsts_db, const char *host, uint16_t port);
 typedef wget_hsts_db *wget_hsts_db_init_fn(wget_hsts_db *hsts_db, const char *fname);
 typedef void wget_hsts_db_deinit_fn(wget_hsts_db *hsts_db);
 typedef void wget_hsts_db_free_fn(wget_hsts_db **hsts_db);
-typedef void wget_hsts_db_add_fn(wget_hsts_db *hsts_db, const char *host, uint16_t port, time_t maxage, int include_subdomains);
+typedef void wget_hsts_db_add_fn(wget_hsts_db *hsts_db, const char *host, uint16_t port, int64_t maxage, bool include_subdomains);
 typedef int wget_hsts_db_save_fn(wget_hsts_db *hsts_db);
 typedef int wget_hsts_db_load_fn(wget_hsts_db *hsts_db);
 typedef void wget_hsts_db_set_fname_fn(wget_hsts_db *hsts_db, const char *fname);
@@ -1515,7 +1519,7 @@ WGETAPI void
 WGETAPI void
 	wget_hpkp_set_host(wget_hpkp *hpkp, const char *host);
 WGETAPI void
-	wget_hpkp_set_maxage(wget_hpkp *hpkp, time_t maxage);
+	wget_hpkp_set_maxage(wget_hpkp *hpkp, int64_t maxage);
 WGETAPI void
 	wget_hpkp_set_include_subdomains(wget_hpkp *hpkp, bool include_subdomains);
 WGETAPI int
@@ -1526,7 +1530,7 @@ WGETAPI void
 	wget_hpkp_get_pins(wget_hpkp *hpkp, const char **pin_types, size_t *sizes, const void **pins);
 WGETAPI const char *
 	wget_hpkp_get_host(wget_hpkp *hpkp);
-WGETAPI time_t
+WGETAPI int64_t
 	wget_hpkp_get_maxage(wget_hpkp *hpkp);
 WGETAPI bool
 	wget_hpkp_get_include_subdomains(wget_hpkp *hpkp);
@@ -1558,7 +1562,7 @@ WGETAPI void
 WGETAPI void
 	wget_tls_session_free(wget_tls_session *tls_session);
 WGETAPI wget_tls_session * NULLABLE
-	wget_tls_session_new(const char *host, time_t maxage, const void *data, size_t data_size);
+	wget_tls_session_new(const char *host, int64_t maxage, const void *data, size_t data_size);
 WGETAPI int
 	wget_tls_session_get(const wget_tls_session_db *tls_session_db, const char *host, void **data, size_t *size);
 WGETAPI wget_tls_session_db * NULLABLE
@@ -1599,8 +1603,8 @@ typedef void wget_ocsp_db_deinit_fn(wget_ocsp_db *ocsp_db);
 typedef void wget_ocsp_db_free_fn(wget_ocsp_db **ocsp_db);
 typedef bool wget_ocsp_fingerprint_in_cache_fn(const wget_ocsp_db *ocsp_db, const char *fingerprint, int *valid);
 typedef bool wget_ocsp_hostname_is_valid_fn(const wget_ocsp_db *ocsp_db, const char *hostname);
-typedef void wget_ocsp_db_add_fingerprint_fn(wget_ocsp_db *ocsp_db, const char *fingerprint, time_t maxage, bool valid);
-typedef void wget_ocsp_db_add_host_fn(wget_ocsp_db *ocsp_db, const char *host, time_t maxage);
+typedef void wget_ocsp_db_add_fingerprint_fn(wget_ocsp_db *ocsp_db, const char *fingerprint, int64_t maxage, bool valid);
+typedef void wget_ocsp_db_add_host_fn(wget_ocsp_db *ocsp_db, const char *host, int64_t maxage);
 typedef int wget_ocsp_db_save_fn(wget_ocsp_db *ocsp_db);
 typedef int wget_ocsp_db_load_fn(wget_ocsp_db *ocsp_db);
 
@@ -1743,17 +1747,19 @@ typedef struct {
 typedef struct {
 	wget_string
 		url; //!< URL within the parsed document (pointer and length)
+	wget_string
+		download; //!< Value of additional 'download' attribute, the name to be saved to disk
 	char
 		attr[16]; //!< name of the attribute containing the URL, e.g. 'href'
 	char
-		dir[16]; //!< name of the HTML tag containing the URL, e.g. 'a'
+		tag[16]; //!< name of the HTML tag containing the URL, e.g. 'a'
 	bool
 		link_inline : 1; //!< 1 = rel was 'stylesheet' or 'shortcut icon'
 } wget_html_parsed_url;
 
 typedef struct {
 	wget_vector
-		*uris; //!< list of found URLs (entries: wget_html_parsed_url_t)
+		*uris; //!< list of found URLs (entries: wget_html_parsed_url)
 	const char *
 		encoding; //!< the charset encoding set by the parsed document or NULL if none
 	wget_string
@@ -1899,12 +1905,12 @@ WGETAPI int
 WGETAPI void
 	wget_tcp_set_connect_timeout(wget_tcp *tcp, int timeout);
 WGETAPI void
-	wget_tcp_set_tcp_fastopen(wget_tcp *tcp, int tcp_fastopen);
+	wget_tcp_set_tcp_fastopen(wget_tcp *tcp, bool tcp_fastopen);
 WGETAPI void
-	wget_tcp_set_tls_false_start(wget_tcp *tcp, int false_start);
+	wget_tcp_set_tls_false_start(wget_tcp *tcp, bool false_start);
 WGETAPI void
-	wget_tcp_set_ssl(wget_tcp *tcp, int ssl);
-WGETAPI int
+	wget_tcp_set_ssl(wget_tcp *tcp, bool ssl);
+WGETAPI bool
 	wget_tcp_get_ssl(wget_tcp *tcp) WGET_GCC_PURE;
 WGETAPI const char * NULLABLE
 	wget_tcp_get_ip(wget_tcp *tcp) WGET_GCC_PURE;
@@ -1916,9 +1922,9 @@ WGETAPI void
 	wget_tcp_set_ssl_ca_file(wget_tcp *tcp, const char *cafile);
 WGETAPI void
 	wget_tcp_set_ssl_key_file(wget_tcp *tcp, const char *certfile, const char *keyfile);
-WGETAPI char
+WGETAPI bool
 	wget_tcp_get_tcp_fastopen(wget_tcp *tcp) WGET_GCC_PURE;
-WGETAPI char
+WGETAPI bool
 	wget_tcp_get_tls_false_start(wget_tcp *tcp) WGET_GCC_PURE;
 WGETAPI int
 	wget_tcp_get_family(wget_tcp *tcp) WGET_GCC_PURE;
@@ -1938,6 +1944,8 @@ WGETAPI void
 	wget_tcp_set_protocol(wget_tcp *tcp, int protocol);
 WGETAPI void
 	wget_tcp_set_bind_address(wget_tcp *tcp, const char *bind_address);
+WGETAPI void
+	wget_tcp_set_bind_interface(wget_tcp *tcp, const char *bind_interface);
 WGETAPI int
 	wget_tcp_connect(wget_tcp *tcp, const char *host, uint16_t port);
 WGETAPI int
@@ -1945,9 +1953,9 @@ WGETAPI int
 WGETAPI void
 	wget_tcp_tls_stop(wget_tcp *tcp);
 WGETAPI ssize_t
-	wget_tcp_vprintf(wget_tcp *tcp, const char *fmt, va_list args) WGET_GCC_PRINTF_FORMAT(2,0);
+	wget_tcp_vprintf(wget_tcp *tcp, const char *restrict fmt, va_list args) WGET_GCC_PRINTF_FORMAT(2,0);
 WGETAPI ssize_t
-	wget_tcp_printf(wget_tcp *tcp, const char *fmt, ...) WGET_GCC_PRINTF_FORMAT(2,3);
+	wget_tcp_printf(wget_tcp *tcp, const char *restrict fmt, ...) WGET_GCC_PRINTF_FORMAT(2,3);
 WGETAPI ssize_t
 	wget_tcp_write(wget_tcp *tcp, const char *buf, size_t count);
 WGETAPI ssize_t
@@ -2106,6 +2114,8 @@ typedef struct {
 		method[8]; //!< currently we just need HEAD, GET and POST
 	bool
 		response_keepheader : 1; //!< the application wants the response header data
+	bool
+		response_ignorelength : 1; //!< ignore the Content-Length in the response header
 	bool
 		debug_skip_body : 1; //!< if set, do not print the request body (e.g. because it's binary)
 	long long
@@ -2145,14 +2155,16 @@ struct wget_http_response_st {
 		header; //!< the raw header data if requested by the application
 	wget_buffer *
 		body; //!< the body data
+	long long
+		response_end; //!< when this response was received
 	size_t
 		content_length; //!< length of the body data
 	size_t
 		cur_downloaded,
 		accounted_for;	// reported to bar
-	time_t
+	int64_t
 		last_modified;
-	time_t
+	int64_t
 		hsts_maxage;
 	char
 		reason[32]; //!< reason string after the status code
@@ -2167,15 +2179,15 @@ struct wget_http_response_st {
 	wget_transfer_encoding
 		transfer_encoding;
 	char
-		content_encoding,
+		content_encoding;
+	bool
 		hsts_include_subdomains,
 		keep_alive;
 	bool
 		content_length_valid : 1,
+		length_inconsistent : 1, //!< set when length of data received is not same as Content-Length
 		hsts : 1, //!< if hsts_maxage and hsts_include_subdomains are valid
 		csp : 1;
-	long long
-		response_end; //!< when this response was received
 };
 
 typedef struct wget_http_connection_st wget_http_connection;
@@ -2204,7 +2216,7 @@ WGETAPI const char *
 	wget_http_parse_name(const char *s, const char **name) WGET_GCC_NONNULL_ALL;
 WGETAPI const char *
 	wget_parse_name_fixed(const char *s, const char **name, size_t *namelen) WGET_GCC_NONNULL_ALL;
-WGETAPI time_t
+WGETAPI int64_t
 	wget_http_parse_full_date(const char *s) WGET_GCC_NONNULL_ALL;
 WGETAPI const char *
 	wget_http_parse_link(const char *s, wget_http_link *link) WGET_GCC_NONNULL_ALL;
@@ -2225,25 +2237,25 @@ WGETAPI const char *
 WGETAPI const char *
 	wget_http_parse_content_disposition(const char *s, const char **filename) WGET_GCC_NONNULL((1));
 WGETAPI const char *
-	wget_http_parse_strict_transport_security(const char *s, time_t *maxage, char *include_subdomains) WGET_GCC_NONNULL((1));
+	wget_http_parse_strict_transport_security(const char *s, int64_t *maxage, bool *include_subdomains) WGET_GCC_NONNULL_ALL;
 WGETAPI const char *
 	wget_http_parse_public_key_pins(const char *s, wget_hpkp *hpkp) WGET_GCC_NONNULL((1));
 WGETAPI const char *
-	wget_http_parse_connection(const char *s, char *keep_alive) WGET_GCC_NONNULL_ALL;
+	wget_http_parse_connection(const char *s, bool *keep_alive) WGET_GCC_NONNULL_ALL;
 WGETAPI const char *
 	wget_http_parse_setcookie(const char *s, wget_cookie **cookie) WGET_GCC_NONNULL((1));
 WGETAPI const char *
 	wget_http_parse_etag(const char *s, const char **etag) WGET_GCC_NONNULL((1));
 
 WGETAPI char *
-	wget_http_print_date(time_t t, char *buf, size_t bufsize) WGET_GCC_NONNULL_ALL;
+	wget_http_print_date(int64_t t, char *buf, size_t bufsize) WGET_GCC_NONNULL_ALL;
 
 WGETAPI void
 	wget_http_add_param(wget_vector **params, wget_http_header_param *param) WGET_GCC_NONNULL_ALL;
 WGETAPI int
-	wget_http_add_header_vprintf(wget_http_request *req, const char *name, const char *fmt, va_list args) WGET_GCC_PRINTF_FORMAT(3,0) WGET_GCC_NONNULL_ALL;
+	wget_http_add_header_vprintf(wget_http_request *req, const char *name, const char *restrict fmt, va_list args) WGET_GCC_PRINTF_FORMAT(3,0) WGET_GCC_NONNULL_ALL;
 WGETAPI int
-	wget_http_add_header_printf(wget_http_request *req, const char *name, const char *fmt, ...) WGET_GCC_PRINTF_FORMAT(3,4) WGET_GCC_NONNULL((1,2,3));
+	wget_http_add_header_printf(wget_http_request *req, const char *name, const char *restrict fmt, ...) WGET_GCC_PRINTF_FORMAT(3,4) WGET_GCC_NONNULL((1,2,3));
 WGETAPI int
 	wget_http_add_header(wget_http_request *req, const char *name, const char *value) WGET_GCC_NONNULL_ALL;
 WGETAPI int
@@ -2402,6 +2414,7 @@ WGETAPI int
  * Hash convenience routines
  */
 
+// don't use 'restrict' here as out, fmt and argument pointers may overlap
 WGETAPI void WGET_GCC_PRINTF_FORMAT(4,5) WGET_GCC_NONNULL_ALL
 	wget_hash_printf_hex(wget_digest_algorithm algorithm, char *out, size_t outsize, const char *fmt, ...);
 
@@ -2411,7 +2424,7 @@ WGETAPI void WGET_GCC_PRINTF_FORMAT(4,5) WGET_GCC_NONNULL_ALL
  */
 
 typedef struct {
-	wget_iri
+	const wget_iri
 		*iri;        //!< parsed URL of the mirror
 	int
 		priority;    //!< priority of the mirror
@@ -2439,9 +2452,9 @@ typedef struct {
 	const char
 		*name;    //!< filename
 	wget_vector
-		*mirrors, //!< mirrors that provide the file (element: wget_metalink_mirror_t)
-		*hashes,  //!< checksums of complete file (element: wget_metalink_hash_t)
-		*pieces;  //!< checksums of smaller pieces of the file (element: wget_metalink_piece_t)
+		*mirrors, //!< mirrors that provide the file (element: wget_metalink_mirror)
+		*hashes,  //!< checksums of complete file (element: wget_metalink_hash)
+		*pieces;  //!< checksums of smaller pieces of the file (element: wget_metalink_piece)
 	off_t
 		size;     //!< total size of the file
 } wget_metalink;
@@ -2493,11 +2506,11 @@ WGETAPI void
 WGETAPI void
 	wget_bar_print(wget_bar *bar, int slot, const char *s);
 WGETAPI void
-	wget_bar_vprintf(wget_bar *bar, int slot, const char *fmt, va_list args) WGET_GCC_PRINTF_FORMAT(3,0) WGET_GCC_NONNULL_ALL;
+	wget_bar_vprintf(wget_bar *bar, int slot, const char *restrict fmt, va_list args) WGET_GCC_PRINTF_FORMAT(3,0) WGET_GCC_NONNULL_ALL;
 WGETAPI void
-	wget_bar_printf(wget_bar *bar, int slot, const char *fmt, ...) WGET_GCC_PRINTF_FORMAT(3,4) WGET_GCC_NONNULL_ALL;
+	wget_bar_printf(wget_bar *bar, int slot, const char *restrict fmt, ...) WGET_GCC_PRINTF_FORMAT(3,4) WGET_GCC_NONNULL_ALL;
 WGETAPI void
-	wget_bar_slot_begin(wget_bar *bar, int slot, const char *filename, int new_file, ssize_t filesize) WGET_GCC_NONNULL_ALL;
+	wget_bar_slot_begin(wget_bar *bar, int slot, const char *filename, int new_file, ssize_t filesize) WGET_GCC_NONNULL((1));
 WGETAPI void
 	wget_bar_slot_downloaded(wget_bar *bar, int slot, size_t nbytes);
 WGETAPI void
@@ -2809,10 +2822,10 @@ typedef struct
 		version,
 		cert_chain_size;
 	char
-		http_protocol,
-		false_start,
-		tfo;
+		http_protocol;
 	bool
+		false_start,
+		tfo,
 		tls_con,
 		resumed;
 } wget_tls_stats_data;
