@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2012 Tim Ruehsen
+ * Copyright (c) 2015-2022 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -35,36 +35,45 @@
 #include <wget.h>
 #include "private.h"
 
-typedef struct _entry_st _entry_t;
+typedef struct entry_st entry_t;
 
-struct _entry_st {
+struct entry_st {
 	void
 		*key,
 		*value;
-	_entry_t
+	entry_t
 		*next;
 	unsigned int
 		hash;
 };
 
-struct _wget_hashmap_st {
-	wget_hashmap_hash_t
-		hash; // hash function
-	wget_hashmap_compare_t
-		cmp; // compare function
-	wget_hashmap_key_destructor_t
-		key_destructor; // key destructor function
-	wget_hashmap_value_destructor_t
-		value_destructor; // value destructor function
-	_entry_t
-		**entry; // pointer to array of pointers to entries
+struct wget_hashmap_st {
+	wget_hashmap_hash_fn
+		*hash; // hash function
+	wget_hashmap_compare_fn
+		*cmp; // compare function
+	wget_hashmap_key_destructor
+		*key_destructor; // key destructor function
+	wget_hashmap_value_destructor
+		*value_destructor; // value destructor function
+	entry_t
+		**entry;   // pointer to array of pointers to entries
 	int
-		max,     // allocated entries
-		cur,     // number of entries in use
-		off,     // resize strategy: >0: resize = max + off, <0: resize = -off * max
+		max,       // allocated entries
+		cur,       // number of entries in use
 		threshold; // resize when max reaches threshold
 	float
-		factor;
+		resize_factor, // resize strategy: >0: resize = off * max, <0: resize = max + (-off)
+		load_factor;
+};
+
+struct wget_hashmap_iterator_st {
+	struct wget_hashmap_st
+		*h;
+	entry_t
+		*entry;
+	int
+		pos;
 };
 
 /**
@@ -76,6 +85,71 @@ struct _wget_hashmap_st {
  * Hashmaps are key/value stores that perform at O(1) for insertion, searching and removing.
  */
 
+/**
+ * \param[in] h Hashmap
+ * \return New iterator instance for \p h
+ *
+ * Creates a hashmap iterator for \p h.
+ */
+wget_hashmap_iterator *wget_hashmap_iterator_alloc(wget_hashmap *h)
+{
+	struct wget_hashmap_iterator_st *iter = wget_calloc(1, sizeof(struct wget_hashmap_iterator_st));
+
+	if (iter)
+		iter->h = h;
+
+	return (wget_hashmap_iterator *) iter;
+}
+
+/**
+ * \param[in] iter Hashmap iterator
+ *
+ * Free the given iterator \p iter.
+ */
+void wget_hashmap_iterator_free(wget_hashmap_iterator **iter)
+{
+	if (iter)
+		xfree(*iter);
+}
+
+/**
+ * \param[in] iter Hashmap iterator
+ * \param[out] value Pointer to the value belonging to the returned key
+ * \return Pointer to the key or NULL if no more elements left
+ *
+ * Returns the next key / value in the hashmap. If all key/value pairs have been
+ * iterated over the function returns NULL and \p value is untouched.
+ *
+ * When iterating over a hashmap, the order of returned key/value pairs is not defined.
+ */
+void *wget_hashmap_iterator_next(wget_hashmap_iterator *iter, void **value)
+{
+	struct wget_hashmap_iterator_st *_iter = (struct wget_hashmap_iterator_st *) iter;
+	struct wget_hashmap_st *h = _iter->h;
+
+	if (_iter->entry) {
+		if ((_iter->entry = _iter->entry->next)) {
+found:
+			if (value)
+				*value = _iter->entry->value;
+			return _iter->entry->key;
+		}
+
+		_iter->pos++;
+	}
+
+	if (!_iter->entry && h) {
+		for (; _iter->pos < h->max; _iter->pos++) {
+			if (h->entry[_iter->pos]) {
+				_iter->entry = h->entry[_iter->pos];
+				goto found;
+			}
+		}
+	}
+
+	return NULL;
+}
+
 /**
  * \param[in] max Initial number of pre-allocated entries
  * \param[in] hash Hash function to build hashes from elements
@@ -90,28 +164,37 @@ struct _wget_hashmap_st {
  * wget_hashmap_hashcmpfunc() with appropriate function pointers. No doing so will result
  * in undefined behavior (likely you'll see a segmentation fault).
  */
-wget_hashmap_t *wget_hashmap_create(int max, wget_hashmap_hash_t hash, wget_hashmap_compare_t cmp)
+wget_hashmap *wget_hashmap_create(int max, wget_hashmap_hash_fn *hash, wget_hashmap_compare_fn *cmp)
 {
-	wget_hashmap_t *h = xmalloc(sizeof(wget_hashmap_t));
+	wget_hashmap *h = wget_malloc(sizeof(wget_hashmap));
+
+	if (!h)
+		return NULL;
+
+	h->entry = wget_calloc(max, sizeof(entry_t *));
+
+	if (!h->entry) {
+		xfree(h);
+		return NULL;
+	}
 
-	h->entry = xcalloc(max, sizeof(_entry_t *));
 	h->max = max;
 	h->cur = 0;
-	h->off = -2;
+	h->resize_factor = 2;
 	h->hash = hash;
 	h->cmp = cmp;
 	h->key_destructor = free;
 	h->value_destructor = free;
-	h->factor = 0.75;
-	h->threshold = (int)(max * h->factor);
+	h->load_factor = 0.75;
+	h->threshold = (int)(max * h->load_factor);
 
 	return h;
 }
 
-G_GNUC_WGET_NONNULL_ALL
-static _entry_t * hashmap_find_entry(const wget_hashmap_t *h, const char *key, unsigned int hash, int pos)
+WGET_GCC_NONNULL_ALL
+static entry_t * hashmap_find_entry(const wget_hashmap *h, const char *key, unsigned int hash)
 {
-	for (_entry_t * e = h->entry[pos]; e; e = e->next) {
+	for (entry_t * e = h->entry[hash % h->max]; e; e = e->next) {
 		if (hash == e->hash && (key == e->key || !h->cmp(key, e->key))) {
 			return e;
 		}
@@ -120,45 +203,43 @@ static _entry_t * hashmap_find_entry(const wget_hashmap_t *h, const char *key, u
 	return NULL;
 }
 
-G_GNUC_WGET_NONNULL_ALL
-static void hashmap_rehash(wget_hashmap_t *h, int newmax, int recalc_hash)
+WGET_GCC_NONNULL_ALL
+static void hashmap_rehash(wget_hashmap *h, entry_t **new_entry, int newmax, int recalc_hash)
 {
-	_entry_t **new_entry, *entry, *next;
+	entry_t *entry, *next;
 	int cur = h->cur;
 
-	if (cur) {
-		int pos;
-		new_entry = xcalloc(newmax, sizeof(_entry_t *));
+	for (int it = 0; it < h->max && cur; it++) {
+		for (entry = h->entry[it]; entry; entry = next) {
+			next = entry->next;
 
-		for (int it = 0; it < h->max && cur; it++) {
-			for (entry = h->entry[it]; entry; entry = next) {
-				next = entry->next;
+			// now move entry from 'h' to 'new_hashmap'
+			if (recalc_hash)
+				entry->hash = h->hash(entry->key);
+			int pos = entry->hash % newmax;
+			entry->next = new_entry[pos];
+			new_entry[pos] = entry;
 
-				// now move entry from 'h' to 'new_hashmap'
-				if (recalc_hash)
-					entry->hash = h->hash(entry->key);
-				pos = entry->hash % newmax;
-				entry->next = new_entry[pos];
-				new_entry[pos] = entry;
-
-				cur--;
-			}
+			cur--;
 		}
-
-		xfree(h->entry);
-		h->entry = new_entry;
-		h->max = newmax;
-		h->threshold = (int)(newmax * h->factor);
 	}
+
+	xfree(h->entry);
+	h->entry = new_entry;
+	h->max = newmax;
+	h->threshold = (int)(newmax * h->load_factor);
 }
 
-G_GNUC_WGET_NONNULL((1,3))
-static void hashmap_new_entry(wget_hashmap_t *h, unsigned int hash, const char *key, const char *value)
+WGET_GCC_NONNULL((1,3))
+static int hashmap_new_entry(wget_hashmap *h, unsigned int hash, const char *key, const char *value)
 {
-	_entry_t *entry;
+	entry_t *entry;
+
+	if (!(entry = wget_malloc(sizeof(entry_t))))
+		return WGET_E_MEMORY;
+
 	int pos = hash % h->max;
 
-	entry = xmalloc(sizeof(_entry_t));
 	entry->key = (void *)key;
 	entry->value = (void *)value;
 	entry->hash = hash;
@@ -166,21 +247,30 @@ static void hashmap_new_entry(wget_hashmap_t *h, unsigned int hash, const char *
 	h->entry[pos] = entry;
 
 	if (++h->cur >= h->threshold) {
-		if (h->off > 0) {
-			hashmap_rehash(h, h->max + h->off, 0);
-		} else if (h->off<-1) {
-			hashmap_rehash(h, h->max * -h->off, 0);
-		} else {
-			// no resizing occurs
+		int newsize = (int) (h->max * h->resize_factor);
+
+		if (newsize > 0) {
+			entry_t **new_entry;
+
+			if (!(new_entry = wget_calloc(newsize, sizeof(entry_t *)))) {
+				h->cur--;
+				xfree(h->entry[pos]);
+				return WGET_E_MEMORY;
+			}
+
+			// h->cur is always > 0 here, so we don't need a check
+			hashmap_rehash(h, new_entry, newsize, 0);
 		}
 	}
+
+	return WGET_E_SUCCESS;
 }
 
 /**
  * \param[in] h Hashmap to put data into
  * \param[in] key Key to insert into \p h
  * \param[in] value Value to insert into \p h
- * \return 0 if inserted a new entry, 1 if entry existed
+ * \return 0 if inserted a new entry, 1 if entry existed, WGET_E_MEMORY if internal allocation failed
  *
  * Insert a key/value pair into hashmap \p h.
  *
@@ -191,16 +281,16 @@ static void hashmap_new_entry(wget_hashmap_t *h, unsigned int hash, const char *
  *
  * To realize a hashset (just keys without values), \p value may be %NULL.
  *
- * Neither \p h nor \p key must be %NULL.
+ * Neither \p h nor \p key must be %NULL, else the return value will always be 0.
  */
-int wget_hashmap_put_noalloc(wget_hashmap_t *h, const void *key, const void *value)
+int wget_hashmap_put(wget_hashmap *h, const void *key, const void *value)
 {
 	if (h && key) {
-		_entry_t *entry;
+		entry_t *entry;
 		unsigned int hash = h->hash(key);
-		int pos = hash % h->max;
+		int rc;
 
-		if ((entry = hashmap_find_entry(h, key, hash, pos))) {
+		if ((entry = hashmap_find_entry(h, key, hash))) {
 			if (entry->key != key && entry->key != value) {
 				if (h->key_destructor)
 					h->key_destructor(entry->key);
@@ -219,52 +309,23 @@ int wget_hashmap_put_noalloc(wget_hashmap_t *h, const void *key, const void *val
 		}
 
 		// a new entry
-		hashmap_new_entry(h, hash, key, value);
+		if ((rc = hashmap_new_entry(h, hash, key, value)) < 0)
+			return rc;
 	}
 
 	return 0;
 }
 
 /**
- * \param[in] h Hashmap to put data into
- * \param[in] key Key to insert into \p h
- * \param[in] keysize Size of \p key
- * \param[in] value Value to insert into \p h
- * \param[in] valuesize Size of \p value
- * \return 0 if inserted a new entry, 1 if entry existed
+ * \param[in] h Hashmap
+ * \param[in] key Key to search for
+ * \return 1 if \p key has been found, 0 if not found
  *
- * Insert a key/value pair into hashmap \p h.
- *
- * If \p key already exists it will not be cloned. In this case the value destructor function
- * will be called with the old value and the new value will be shallow cloned.
- *
- * If \p doesn't exist, both \p key and \p value will be shallow cloned.
- *
- * To realize a hashset (just keys without values), \p value may be %NULL.
- *
- * Neither \p h nor \p key must be %NULL.
+ * Check if \p key exists in \p h.
  */
-int wget_hashmap_put(wget_hashmap_t *h, const void *key, size_t keysize, const void *value, size_t valuesize)
+int wget_hashmap_contains(const wget_hashmap *h, const void *key)
 {
-	if (h && key) {
-		_entry_t *entry;
-		unsigned int hash = h->hash(key);
-		int pos = hash % h->max;
-
-		if ((entry = hashmap_find_entry(h, key, hash, pos))) {
-			if (h->value_destructor)
-				h->value_destructor(entry->value);
-
-			entry->value = wget_memdup(value, valuesize);
-
-			return 1;
-		}
-
-		// a new entry
-		hashmap_new_entry(h, hash, wget_memdup(key, keysize), wget_memdup(value, valuesize));
-	}
-
-	return 0;
+	return wget_hashmap_get(h, key, NULL);
 }
 
 /**
@@ -275,20 +336,17 @@ int wget_hashmap_put(wget_hashmap_t *h, const void *key, size_t keysize, const v
  *
  * Get the value for a given key.
  *
- * If there are %NULL values in the hashmap, you should use this function
- * to distinguish between 'not found' and 'found %NULL value'.
- *
  * Neither \p h nor \p key must be %NULL.
  */
-int wget_hashmap_get_null(const wget_hashmap_t *h, const void *key, void **value)
+#undef wget_hashmap_get
+int wget_hashmap_get(const wget_hashmap *h, const void *key, void **value)
 {
 	if (h && key) {
-		_entry_t *entry;
-		unsigned int hash = h->hash(key);
-		int pos = hash % h->max;
+		entry_t *entry;
 
-		if ((entry = hashmap_find_entry(h, key, hash, pos))) {
-			if (value) *value = entry->value;
+		if ((entry = hashmap_find_entry(h, key, h->hash(key)))) {
+			if (value)
+				*value = entry->value;
 			return 1;
 		}
 	}
@@ -296,44 +354,10 @@ int wget_hashmap_get_null(const wget_hashmap_t *h, const void *key, void **value
 	return 0;
 }
 
-/**
- * \param[in] h Hashmap
- * \param[in] key Key to search for
- * \return Found value or %NULL if not found
- *
- * Get the value for a given key.
- *
- * If there are %NULL values in the hashmap, you should use wget_hashmap_get_null()
- * to distinguish between 'not found' and 'found %NULL value'.
- *
- * Neither \p h nor \p key must be %NULL.
- */
-void *wget_hashmap_get(const wget_hashmap_t *h, const void *key)
+WGET_GCC_NONNULL_ALL
+static int hashmap_remove_entry(wget_hashmap *h, const char *key, int free_kv)
 {
-	void *value;
-
-	if (wget_hashmap_get_null(h, key, &value))
-		return value;
-
-	return NULL;
-}
-
-/**
- * \param[in] h Hashmap
- * \param[in] key Key to search for
- * \return 1 if \p key has been found, 0 if not found
- *
- * Check if \p key exists in \p h.
- */
-int wget_hashmap_contains(const wget_hashmap_t *h, const void *key)
-{
-	return wget_hashmap_get_null(h, key, NULL);
-}
-
-G_GNUC_WGET_NONNULL_ALL
-static int hashmap_remove_entry(wget_hashmap_t *h, const char *key, int free_kv)
-{
-	_entry_t *entry, *next, *prev = NULL;
+	entry_t *entry, *next, *prev = NULL;
 	unsigned int hash = h->hash(key);
 	int pos = hash % h->max;
 
@@ -376,7 +400,7 @@ static int hashmap_remove_entry(wget_hashmap_t *h, const char *key, int free_kv)
  * If \p key is found, the key and value destructor functions are called
  * when removing the entry from the hashmap.
  */
-int wget_hashmap_remove(wget_hashmap_t *h, const void *key)
+int wget_hashmap_remove(wget_hashmap *h, const void *key)
 {
 	if (h && key)
 		return hashmap_remove_entry(h, key, 1);
@@ -393,7 +417,7 @@ int wget_hashmap_remove(wget_hashmap_t *h, const void *key)
  *
  * Key and value destructor functions are *not* called when removing the entry from the hashmap.
  */
-int wget_hashmap_remove_nofree(wget_hashmap_t *h, const void *key)
+int wget_hashmap_remove_nofree(wget_hashmap *h, const void *key)
 {
 	if (h && key)
 		return hashmap_remove_entry(h, key, 0);
@@ -408,7 +432,7 @@ int wget_hashmap_remove_nofree(wget_hashmap_t *h, const void *key)
  *
  * Key and value destructor functions are called for each entry in the hashmap.
  */
-void wget_hashmap_free(wget_hashmap_t **h)
+void wget_hashmap_free(wget_hashmap **h)
 {
 	if (h && *h) {
 		wget_hashmap_clear(*h);
@@ -424,10 +448,10 @@ void wget_hashmap_free(wget_hashmap_t **h)
  *
  * Key and value destructor functions are called for each entry in the hashmap.
  */
-void wget_hashmap_clear(wget_hashmap_t *h)
+void wget_hashmap_clear(wget_hashmap *h)
 {
 	if (h) {
-		_entry_t *entry, *next;
+		entry_t *entry, *next;
 		int it, cur = h->cur;
 
 		for (it = 0; it < h->max && cur; it++) {
@@ -459,7 +483,7 @@ void wget_hashmap_clear(wget_hashmap_t *h)
  *
  * Return the number of entries in the hashmap \p h.
  */
-int wget_hashmap_size(const wget_hashmap_t *h)
+int wget_hashmap_size(const wget_hashmap *h)
 {
 	return h ? h->cur : 0;
 }
@@ -477,10 +501,10 @@ int wget_hashmap_size(const wget_hashmap_t *h)
  *
  * The return value of the last call to \p browse is returned or 0 if either \p h or \p browse is %NULL.
  */
-int wget_hashmap_browse(const wget_hashmap_t *h, wget_hashmap_browse_t browse, void *ctx)
+int wget_hashmap_browse(const wget_hashmap *h, wget_hashmap_browse_fn *browse, void *ctx)
 {
 	if (h && browse) {
-		_entry_t *entry;
+		entry_t *entry;
 		int it, ret, cur = h->cur;
 
 		for (it = 0; it < h->max && cur; it++) {
@@ -501,7 +525,7 @@ int wget_hashmap_browse(const wget_hashmap_t *h, wget_hashmap_browse_t browse, v
  *
  * Set the comparison function.
  */
-void wget_hashmap_setcmpfunc(wget_hashmap_t *h, wget_hashmap_compare_t cmp)
+void wget_hashmap_setcmpfunc(wget_hashmap *h, wget_hashmap_compare_fn *cmp)
 {
 	if (h)
 		h->cmp = cmp;
@@ -510,18 +534,30 @@ void wget_hashmap_setcmpfunc(wget_hashmap_t *h, wget_hashmap_compare_t cmp)
 /**
  * \param[in] h Hashmap
  * \param[in] hash Hash function used to hash keys
+ * \return WGET_E_SUCCESS if set successfully, else WGET_E_MEMORY or WGET_E_INVALID
  *
  * Set the key hash function.
  *
- * The keys of all entries in the hashmap will be hashed again.
+ * The keys of all entries in the hashmap will be hashed again. This includes a memory allocation, so
+ * there is a possibility of failure.
  */
-void wget_hashmap_sethashfunc(wget_hashmap_t *h, wget_hashmap_hash_t hash)
+int wget_hashmap_sethashfunc(wget_hashmap *h, wget_hashmap_hash_fn *hash)
 {
-	if (h) {
-		h->hash = hash;
+	if (!h)
+		return WGET_E_INVALID;
 
-		hashmap_rehash(h, h->max, 1);
-	}
+	if (!h->cur)
+		return WGET_E_SUCCESS; // no re-hashing needed
+
+	entry_t **new_entry = wget_calloc(h->max, sizeof(entry_t *));
+
+	if (!new_entry)
+		return WGET_E_MEMORY;
+
+	h->hash = hash;
+	hashmap_rehash(h, new_entry, h->max, 1);
+
+	return WGET_E_SUCCESS;
 }
 
 /**
@@ -532,7 +568,7 @@ void wget_hashmap_sethashfunc(wget_hashmap_t *h, wget_hashmap_hash_t hash)
  *
  * Default is free().
  */
-void wget_hashmap_set_key_destructor(wget_hashmap_t *h, wget_hashmap_key_destructor_t destructor)
+void wget_hashmap_set_key_destructor(wget_hashmap *h, wget_hashmap_key_destructor *destructor)
 {
 	if (h)
 		h->key_destructor = destructor;
@@ -546,7 +582,7 @@ void wget_hashmap_set_key_destructor(wget_hashmap_t *h, wget_hashmap_key_destruc
  *
  * Default is free().
  */
-void wget_hashmap_set_value_destructor(wget_hashmap_t *h, wget_hashmap_value_destructor_t destructor)
+void wget_hashmap_set_value_destructor(wget_hashmap *h, wget_hashmap_value_destructor *destructor)
 {
 	if (h)
 		h->value_destructor = destructor;
@@ -567,29 +603,31 @@ void wget_hashmap_set_value_destructor(wget_hashmap_t *h, wget_hashmap_value_des
  *
  * Default is 0.75.
  */
-void wget_hashmap_setloadfactor(wget_hashmap_t *h, float factor)
+void wget_hashmap_set_load_factor(wget_hashmap *h, float factor)
 {
 	if (h) {
-		h->factor = factor;
-		h->threshold = (int)(h->max * h->factor);
+		h->load_factor = factor;
+		h->threshold = (int)(h->max * h->load_factor);
 		// rehashing occurs earliest on next put()
 	}
 }
 
 /**
  * \param[in] h Hashmap
- * \param[in] off Hashmap growth mode:
- *   positive values: increase size by \p off entries on each resize
- *   negative values: increase size by multiplying \p -off, e.g. -2 doubles the size on each resize
+ * \param[in] factor Hashmap growth factor
  *
- * Set the growth policy for internal memory.
+ * Set the factor for resizing the hashmap when it's load factor is reached.
  *
- * Default is -2.
+ * The new size is 'factor * oldsize'. If the new size is less or equal 0,
+ * the involved put function will do nothing and the internal state of
+ * the hashmap will not change.
+ *
+ * Default is 2.
  */
-void wget_hashmap_set_growth_policy(wget_hashmap_t *h, int off)
+void wget_hashmap_set_resize_factor(wget_hashmap *h, float factor)
 {
 	if (h)
-		h->off = off;
+		h->resize_factor = factor;
 }
 
 /**@}*/
