@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2013 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2013 Tim Ruehsen
+ * Copyright (c) 2015-2022 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -35,14 +35,14 @@
 #include "private.h"
 #include "http.h"
 
-static int _stream_callback(wget_http_response_t *resp G_GNUC_WGET_UNUSED, void *user_data, const char *data, size_t length)
+static int stream_callback(wget_http_response *resp WGET_GCC_UNUSED, void *user_data, const char *data, size_t length)
 {
 	FILE *stream = (FILE *) user_data;
 
 	size_t nbytes = fwrite(data, 1, length, stream);
 
 	if (nbytes != length) {
-		error_printf(_("Failed to write %zu bytes of data (%d: %s)\n"), length, errno, strerror(errno));
+		error_printf(_("Failed to write %zu bytes of data (%d)\n"), length, errno);
 
 		if (feof(stream))
 			return -1;
@@ -51,30 +51,30 @@ static int _stream_callback(wget_http_response_t *resp G_GNUC_WGET_UNUSED, void
 	return 0;
 }
 
-static int _fd_callback(wget_http_response_t *resp G_GNUC_WGET_UNUSED, void *user_data, const char *data, size_t length)
+static int fd_callback(wget_http_response *resp WGET_GCC_UNUSED, void *user_data, const char *data, size_t length)
 {
 	int fd = *(int *) user_data;
 	ssize_t nbytes = write(fd, data, length);
 
 	if (nbytes == -1 || (size_t) nbytes != length)
-		error_printf(_("Failed to write %zu bytes of data (%d: %s)\n"), length, errno, strerror(errno));
+		error_printf(_("Failed to write %zu bytes of data (%d)\n"), length, errno);
 
 	return 0;
 }
 
-wget_http_response_t *wget_http_get(int first_key, ...)
+wget_http_response *wget_http_get(int first_key, ...)
 {
-	wget_vector_t *headers = wget_vector_create(8, NULL);
-	wget_iri_t *uri = NULL;
-	wget_http_connection_t *conn = NULL, **connp = NULL;
-	wget_http_request_t *req;
-	wget_http_response_t *resp = NULL;
-	wget_vector_t *challenges = NULL;
-	wget_cookie_db_t *cookie_db = NULL;
+	wget_vector *headers;
+	wget_iri *uri = NULL;
+	wget_http_connection *conn = NULL, **connp = NULL;
+	wget_http_request *req;
+	wget_http_response *resp = NULL;
+	wget_vector *challenges = NULL;
+	wget_cookie_db *cookie_db = NULL;
 	FILE *saveas_stream = NULL;
-	wget_http_body_callback_t saveas_callback = NULL;
+	wget_http_body_callback *saveas_callback = NULL;
 	int saveas_fd = -1;
-	wget_http_header_callback_t header_callback = NULL;
+	wget_http_header_callback *header_callback = NULL;
 	va_list args;
 	const char *url = NULL,	*url_encoding = NULL, *scheme = "GET";
 	const char *http_username = NULL, *http_password = NULL;
@@ -90,9 +90,14 @@ wget_http_response_t *wget_http_get(int first_key, ...)
 			keep_header : 1,
 			free_uri : 1;
 	} bits = {
-		.cookies_enabled = !!wget_global_get_int(WGET_COOKIES_ENABLED)
+		.cookies_enabled = wget_global_get_int(WGET_COOKIES_ENABLED) != 0
 	};
 
+	if (!(headers = wget_vector_create(8, NULL))) {
+		debug_printf("no memory\n");
+		return NULL;
+	}
+
 	va_start(args, first_key);
 	for (key = first_key; key; key = va_arg(args, int)) {
 		switch (key) {
@@ -100,22 +105,25 @@ wget_http_response_t *wget_http_get(int first_key, ...)
 			url = va_arg(args, const char *);
 			break;
 		case WGET_HTTP_URI:
-			uri = va_arg(args, wget_iri_t *);
+			uri = va_arg(args, wget_iri *);
 			break;
 		case WGET_HTTP_URL_ENCODING:
 			url_encoding = va_arg(args, const char *);
 			break;
 		case WGET_HTTP_HEADER_ADD:
 		{
-			wget_http_header_param_t param = {
+			wget_http_header_param param = {
 				.name = va_arg(args, const char *),
 				.value = va_arg(args, const char *)
 			};
-			wget_vector_add(headers, &param, sizeof(param));
+			if (wget_vector_add_memdup(headers, &param, sizeof(param)) < 0) {
+				va_end(args);
+				goto out;
+			}
 			break;
 		}
 		case WGET_HTTP_CONNECTION_PTR:
-			connp = va_arg(args, wget_http_connection_t **);
+			connp = va_arg(args, wget_http_connection **);
 			if (connp)
 				conn = *connp;
 			break;
@@ -132,14 +140,14 @@ wget_http_response_t *wget_http_get(int first_key, ...)
 			saveas_stream = va_arg(args, FILE *);
 			break;
 		case WGET_HTTP_BODY_SAVEAS_FUNC:
-			saveas_callback = va_arg(args, wget_http_body_callback_t);
+			saveas_callback = va_arg(args, wget_http_body_callback *);
 			body_user_data = va_arg(args, void *);
 			break;
 		case WGET_HTTP_BODY_SAVEAS_FD:
 			saveas_fd = va_arg(args, int);
 			break;
 		case WGET_HTTP_HEADER_FUNC:
-			header_callback = va_arg(args, wget_http_header_callback_t);
+			header_callback = va_arg(args, wget_http_header_callback *);
 			header_user_data = va_arg(args, void *);
 			break;
 		case WGET_HTTP_SCHEME:
@@ -151,6 +159,7 @@ wget_http_response_t *wget_http_get(int first_key, ...)
 			break;
 		default:
 			error_printf(_("Unknown option %d\n"), key);
+			va_end(args);
 			goto out;
 		}
 	}
@@ -171,12 +180,14 @@ wget_http_response_t *wget_http_get(int first_key, ...)
 	}
 
 	if (bits.cookies_enabled)
-		cookie_db = (wget_cookie_db_t *)wget_global_get_ptr(WGET_COOKIE_DB);
+		cookie_db = (wget_cookie_db *)wget_global_get_ptr(WGET_COOKIE_DB);
 
 	while (uri && redirection_level <= max_redirections) {
 		// create a HTTP/1.1 GET request.
 		// the only default header is 'Host: domain' (taken from uri)
 		req = wget_http_create_request(uri, scheme);
+		if (!req)
+			goto out;
 
 		// add HTTP headers
 		for (it = 0; it < wget_vector_size(headers); it++) {
@@ -236,20 +247,20 @@ wget_http_response_t *wget_http_get(int first_key, ...)
 				if (saveas_name) {
 					FILE *fp;
 					if ((fp = fopen(saveas_name, "wb"))) {
-						wget_http_request_set_body_cb(req, _stream_callback, fp);
+						wget_http_request_set_body_cb(req, stream_callback, fp);
 						resp = wget_http_get_response(conn);
 						fclose(fp);
 					} else
 						debug_printf("Failed to open '%s' for writing\n", saveas_name);
 				}
 				else if (saveas_stream)  {
-					wget_http_request_set_body_cb(req, _stream_callback, saveas_stream);
+					wget_http_request_set_body_cb(req, stream_callback, saveas_stream);
 					resp = wget_http_get_response(conn);
 				} else if (saveas_callback) {
 					wget_http_request_set_body_cb(req, saveas_callback, body_user_data);
 					resp = wget_http_get_response(conn);
 				} else if (saveas_fd != -1) {
-					wget_http_request_set_body_cb(req, _fd_callback, &saveas_fd);
+					wget_http_request_set_body_cb(req, fd_callback, &saveas_fd);
 					resp = wget_http_get_response(conn);
 				} else
 					resp = wget_http_get_response(conn);
@@ -291,11 +302,11 @@ wget_http_response_t *wget_http_get(int first_key, ...)
 
 		if (resp->location) {
 			char uri_sbuf[1024];
-			wget_buffer_t uri_buf;
+			wget_buffer uri_buf;
 
 			// if relative location, convert to absolute
 			wget_buffer_init(&uri_buf, uri_sbuf, sizeof(uri_sbuf));
-			wget_iri_relative_to_abs(uri, resp->location, strlen(resp->location), &uri_buf);
+			wget_iri_relative_to_abs(uri, resp->location, -1, &uri_buf);
 
 			if (bits.free_uri)
 				wget_iri_free(&uri);
