@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2013 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2013 Tim Ruehsen
+ * Copyright (c) 2015-2022 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -32,23 +32,25 @@
 #include <wget.h>
 #include "private.h"
 
-static struct _CONFIG {
-	char *
-		cookie_file;
-	wget_cookie_db_t *
-		cookie_db;
+static struct config {
 	char
+		*cookie_file;
+	wget_cookie_db
+		*cookie_db;
+	bool
 		cookies_enabled,
 		keep_session_cookies;
-} _config = {
+} config = {
 	.cookies_enabled = 0
 };
 
-static int _init;
-static wget_thread_mutex_t _mutex;
+static wget_dns_cache *dns_cache;
+
+static int global_initialized;
+static wget_thread_mutex _mutex;
 static bool initialized;
 
-static void __attribute__ ((constructor)) _wget_global_init(void)
+static void __attribute__ ((constructor)) global_init(void)
 {
 	if (!initialized) {
 		wget_thread_mutex_init(&_mutex);
@@ -56,7 +58,7 @@ static void __attribute__ ((constructor)) _wget_global_init(void)
 	}
 }
 
-static void __attribute__ ((destructor)) _wget_global_exit(void)
+static void __attribute__ ((destructor)) global_exit(void)
 {
 	if (initialized) {
 		wget_thread_mutex_destroy(&_mutex);
@@ -73,23 +75,22 @@ static void __attribute__ ((destructor)) _wget_global_exit(void)
 void wget_global_init(int first_key, ...)
 {
 	va_list args;
-	int key, rc;
+	int key, rc, caching;
 	const char *psl_file = NULL;
-	wget_logger_func_t func; // intermediate var to satisfy MSVC
+	wget_logger_func *func; // intermediate var to satisfy MSVC
 
 	// just in case that automatic initializers didn't work,
 	// e.g. maybe a static build
-	_wget_global_init();
+	global_init();
 
 	wget_thread_mutex_lock(_mutex);
 
-	if (_init++) {
+	if (global_initialized++) {
 		wget_thread_mutex_unlock(_mutex);
 		return;
 	}
 
 	wget_console_init();
-	wget_dns_cache_init();
 	wget_random_init();
 	wget_http_init();
 
@@ -100,7 +101,7 @@ void wget_global_init(int first_key, ...)
 			wget_logger_set_stream(wget_get_logger(WGET_LOGGER_DEBUG), va_arg(args, FILE *));
 			break;
 		case WGET_DEBUG_FUNC:
-			func = va_arg(args, wget_logger_func_t);
+			func = va_arg(args, wget_logger_func *);
 			wget_logger_set_func(wget_get_logger(WGET_LOGGER_DEBUG), func);
 			break;
 		case WGET_DEBUG_FILE:
@@ -110,7 +111,7 @@ void wget_global_init(int first_key, ...)
 			wget_logger_set_stream(wget_get_logger(WGET_LOGGER_ERROR), va_arg(args, FILE *));
 			break;
 		case WGET_ERROR_FUNC:
-			func = va_arg(args, wget_logger_func_t);
+			func = va_arg(args, wget_logger_func *);
 			wget_logger_set_func(wget_get_logger(WGET_LOGGER_ERROR), func);
 			break;
 		case WGET_ERROR_FILE:
@@ -120,32 +121,38 @@ void wget_global_init(int first_key, ...)
 			wget_logger_set_stream(wget_get_logger(WGET_LOGGER_INFO), va_arg(args, FILE *));
 			break;
 		case WGET_INFO_FUNC:
-			func = va_arg(args, wget_logger_func_t);
+			func = va_arg(args, wget_logger_func *);
 			wget_logger_set_func(wget_get_logger(WGET_LOGGER_INFO), func);
 			break;
 		case WGET_INFO_FILE:
 			wget_logger_set_file(wget_get_logger(WGET_LOGGER_INFO), va_arg(args, const char *));
 			break;
 		case WGET_DNS_CACHING:
-			wget_tcp_set_dns_caching(NULL, va_arg(args, int));
+			caching = va_arg(args, int);
+			if (caching) {
+				if ((rc = wget_dns_cache_init(&dns_cache)) == WGET_E_SUCCESS)
+					wget_dns_set_cache(NULL, dns_cache);
+				else
+					wget_error_printf(_("Failed to init DNS cache (%d)"), rc);
+			}
 			break;
 		case WGET_TCP_FASTFORWARD:
-			wget_tcp_set_tcp_fastopen(NULL, va_arg(args, int));
+			wget_tcp_set_tcp_fastopen(NULL, va_arg(args, int) != 0);
 			break;
 		case WGET_COOKIE_SUFFIXES:
 			psl_file = va_arg(args, const char *);
-			_config.cookies_enabled = 1;
+			config.cookies_enabled = 1;
 			break;
 		case WGET_COOKIES_ENABLED:
-			_config.cookies_enabled = !!va_arg(args, int);
+			config.cookies_enabled = va_arg(args, int) != 0;
 			break;
 		case WGET_COOKIE_FILE:
 			// load cookie-store
-			_config.cookies_enabled = 1;
-			_config.cookie_file = va_arg(args, char *);
+			config.cookies_enabled = 1;
+			config.cookie_file = va_arg(args, char *);
 			break;
 		case WGET_COOKIE_KEEPSESSIONCOOKIES:
-			_config.keep_session_cookies = !!va_arg(args, int);
+			config.keep_session_cookies = va_arg(args, int) != 0;
 			break;
 		case WGET_BIND_ADDRESS:
 			wget_tcp_set_bind_address(NULL, va_arg(args, const char *));
@@ -156,6 +163,9 @@ void wget_global_init(int first_key, ...)
 		case WGET_NET_FAMILY_PREFERRED:
 			wget_tcp_set_preferred_family(NULL, va_arg(args, int));
 			break;
+		case WGET_BIND_INTERFACE:
+			wget_tcp_set_bind_interface(NULL, va_arg(args, const char *));
+			break;
 		default:
 			wget_thread_mutex_unlock(_mutex);
 			wget_error_printf(_("%s: Unknown option %d"), __func__, key);
@@ -165,11 +175,11 @@ void wget_global_init(int first_key, ...)
 	}
 	va_end(args);
 
-	if (_config.cookies_enabled && _config.cookie_file) {
-		_config.cookie_db = wget_cookie_db_init(NULL);
-		wget_cookie_set_keep_session_cookies(_config.cookie_db, _config.keep_session_cookies);
-		wget_cookie_db_load(_config.cookie_db, _config.cookie_file);
-		wget_cookie_db_load_psl(_config.cookie_db, psl_file);
+	if (config.cookies_enabled && config.cookie_file) {
+		config.cookie_db = wget_cookie_db_init(NULL);
+		wget_cookie_set_keep_session_cookies(config.cookie_db, config.keep_session_cookies);
+		wget_cookie_db_load(config.cookie_db, config.cookie_file);
+		wget_cookie_db_load_psl(config.cookie_db, psl_file);
 	}
 
 	rc = wget_net_init();
@@ -191,17 +201,16 @@ void wget_global_deinit(void)
 
 	// we need to lock a static mutex here
 
-	if (_init == 1) {
+	if (global_initialized == 1) {
 		// free resources here
-		if (_config.cookie_db && _config.cookies_enabled && _config.cookie_file) {
-			wget_cookie_db_save(_config.cookie_db, _config.cookie_file);
-			wget_cookie_db_free(&_config.cookie_db);
+		if (config.cookie_db && config.cookies_enabled && config.cookie_file) {
+			wget_cookie_db_save(config.cookie_db, config.cookie_file);
+			wget_cookie_db_free(&config.cookie_db);
 		}
 		wget_tcp_set_bind_address(NULL, NULL);
-		wget_tcp_set_dns_caching(NULL, 0);
-		wget_dns_cache_free();
 
-		wget_dns_cache_exit();
+		wget_dns_cache_free(&dns_cache);
+
 		rc = wget_net_deinit();
 		wget_ssl_deinit();
 		wget_http_set_http_proxy(NULL, NULL);
@@ -209,9 +218,9 @@ void wget_global_deinit(void)
 		wget_http_set_no_proxy(NULL, NULL);
 	}
 
-	if (_init > 0) _init--;
+	if (global_initialized > 0) global_initialized--;
 
-	_wget_global_exit();
+	global_exit();
 
 	// we need to unlock a static mutex here
 
@@ -224,12 +233,10 @@ void wget_global_deinit(void)
 int wget_global_get_int(int key)
 {
 	switch (key) {
-	case WGET_DNS_CACHING:
-		return wget_tcp_get_dns_caching(NULL);
 	case WGET_COOKIES_ENABLED:
-		return _config.cookies_enabled;
+		return config.cookies_enabled;
 	case WGET_COOKIE_KEEPSESSIONCOOKIES:
-		return _config.keep_session_cookies;
+		return config.keep_session_cookies;
 	case WGET_NET_FAMILY_EXCLUSIVE:
 		return wget_tcp_get_family(NULL);
 	case WGET_NET_FAMILY_PREFERRED:
@@ -256,16 +263,16 @@ const void *wget_global_get_ptr(int key)
 	case WGET_INFO_FILE:
 		return wget_logger_get_file(wget_get_logger(WGET_LOGGER_INFO));
 	case WGET_COOKIE_FILE:
-		return _config.cookie_file;
+		return config.cookie_file;
 	case WGET_COOKIE_DB:
-		return &_config.cookie_db;
+		return config.cookie_db;
 	default:
 		wget_error_printf(_("%s: Unknown option %d"), __func__, key);
 		return NULL;
 	}
 }
 
-wget_global_get_func_t wget_global_get_func(int key)
+wget_global_func *wget_global_get_func(int key)
 {
 	switch (key) {
 	case WGET_DEBUG_FUNC:
