@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2013 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2013 Tim Ruehsen
+ * Copyright (c) 2015-2022 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -35,11 +35,11 @@
 #include "private.h"
 #include "logger.h"
 
-static void G_GNUC_WGET_PRINTF_FORMAT(2,0) G_GNUC_WGET_NONNULL((1,2))
-_logger_vprintf_func(const wget_logger_t *logger, const char *fmt, va_list args)
+static void WGET_GCC_PRINTF_FORMAT(2,0) WGET_GCC_NONNULL((1,2))
+logger_vprintf_func(const wget_logger *logger, const char *fmt, va_list args)
 {
 	char sbuf[4096];
-	wget_buffer_t buf;
+	wget_buffer buf;
 	int err = errno;
 
 	wget_buffer_init(&buf, sbuf, sizeof(sbuf));
@@ -50,16 +50,17 @@ _logger_vprintf_func(const wget_logger_t *logger, const char *fmt, va_list args)
 	errno = err;
 }
 
-static void _logger_write_func(const wget_logger_t *logger, const char *buf, size_t len)
+static void WGET_GCC_NONNULL((1))
+logger_write_func(const wget_logger *logger, const char *buf, size_t len)
 {
 	logger->func(buf, len);
 }
 
-static void  G_GNUC_WGET_PRINTF_FORMAT(2,0) G_GNUC_WGET_NONNULL((1,2))
-_logger_vfprintf(FILE *fp, const char *fmt, va_list args)
+static void  WGET_GCC_PRINTF_FORMAT(2,0) WGET_GCC_NONNULL((1,2))
+logger_vfprintf(FILE *fp, const char *fmt, va_list args)
 {
 	char sbuf[4096];
-	wget_buffer_t buf;
+	wget_buffer buf;
 	int err = errno;
 
 	wget_buffer_init(&buf, sbuf, sizeof(sbuf));
@@ -70,29 +71,31 @@ _logger_vfprintf(FILE *fp, const char *fmt, va_list args)
 	errno = err;
 }
 
-static void  G_GNUC_WGET_PRINTF_FORMAT(2,0) G_GNUC_WGET_NONNULL((1,2))
-_logger_vprintf_file(const wget_logger_t *logger, const char *fmt, va_list args)
+static void  WGET_GCC_PRINTF_FORMAT(2,0) WGET_GCC_NONNULL((1,2))
+logger_vprintf_file(const wget_logger *logger, const char *fmt, va_list args)
 {
-	_logger_vfprintf(logger->fp, fmt, args);
+	logger_vfprintf(logger->fp, fmt, args);
 }
 
-static void _logger_write_file(const wget_logger_t *logger, const char *buf, size_t len)
+static void WGET_GCC_NONNULL((1))
+logger_write_file(const wget_logger *logger, const char *buf, size_t len)
 {
 	fwrite(buf, 1, len, logger->fp);
 }
 
-static void G_GNUC_WGET_PRINTF_FORMAT(2,0) G_GNUC_WGET_NONNULL((1,2))
-_logger_vprintf_fname(const wget_logger_t *logger, const char *fmt, va_list args)
+static void WGET_GCC_PRINTF_FORMAT(2,0) WGET_GCC_NONNULL((1,2))
+logger_vprintf_fname(const wget_logger *logger, const char *fmt, va_list args)
 {
 	FILE *fp = fopen(logger->fname, "a");
 
 	if (fp) {
-		_logger_vfprintf(fp, fmt, args);
+		logger_vfprintf(fp, fmt, args);
 		fclose(fp);
 	}
 }
 
-static void _logger_write_fname(const wget_logger_t *logger, const char *buf, size_t len)
+static void WGET_GCC_NONNULL((1))
+logger_write_fname(const wget_logger *logger, const char *buf, size_t len)
 {
 	FILE *fp = fopen(logger->fname, "a");
 
@@ -102,49 +105,49 @@ static void _logger_write_fname(const wget_logger_t *logger, const char *buf, si
 	}
 }
 
-void wget_logger_set_func(wget_logger_t *logger, wget_logger_func_t func)
+void wget_logger_set_func(wget_logger *logger, wget_logger_func *func)
 {
 	if (logger) {
 		logger->func = func;
-		logger->vprintf = func ? _logger_vprintf_func : NULL;
-		logger->write = func ? _logger_write_func : NULL;
+		logger->vprintf = func ? logger_vprintf_func : NULL;
+		logger->write = func ? logger_write_func : NULL;
 	}
 }
 
-wget_logger_func_t wget_logger_get_func(wget_logger_t *logger)
+wget_logger_func *wget_logger_get_func(wget_logger *logger)
 {
 	return logger ? logger->func : NULL;
 }
 
-void wget_logger_set_stream(wget_logger_t *logger, FILE *fp)
+void wget_logger_set_stream(wget_logger *logger, FILE *fp)
 {
 	if (logger) {
 		logger->fp = fp;
-		logger->vprintf = fp ? _logger_vprintf_file : NULL;
-		logger->write = fp ? _logger_write_file : NULL;
+		logger->vprintf = fp ? logger_vprintf_file : NULL;
+		logger->write = fp ? logger_write_file : NULL;
 	}
 }
 
-FILE *wget_logger_get_stream(wget_logger_t *logger)
+FILE *wget_logger_get_stream(wget_logger *logger)
 {
 	return logger ? logger->fp : NULL;
 }
 
-void wget_logger_set_file(wget_logger_t *logger, const char *fname)
+void wget_logger_set_file(wget_logger *logger, const char *fname)
 {
 	if (logger) {
 		logger->fname = fname;
-		logger->vprintf = fname ? _logger_vprintf_fname : NULL;
-		logger->write = fname ? _logger_write_fname : NULL;
+		logger->vprintf = fname ? logger_vprintf_fname : NULL;
+		logger->write = fname ? logger_write_fname : NULL;
 	}
 }
 
-const char *wget_logger_get_file(wget_logger_t *logger)
+const char *wget_logger_get_file(wget_logger *logger)
 {
 	return logger ? logger->fname : NULL;
 }
 
-bool wget_logger_is_active(wget_logger_t *logger)
+bool wget_logger_is_active(wget_logger *logger)
 {
-	return !!logger->vprintf;
+	return logger->vprintf != 0;
 }
