@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2012 Tim Ruehsen
+ * Copyright (c) 2015-2022 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -43,13 +43,12 @@
 #include <stdlib.h>
 #include <string.h>
 #include <limits.h>
-#include <c-ctype.h>
 
 #include <wget.h>
 #include "private.h"
 
 typedef struct {
-	wget_metalink_t
+	wget_metalink
 		*metalink;
 	int
 		priority;
@@ -60,22 +59,27 @@ typedef struct {
 		location[8];
 	long long
 		length;
-} _metalink_context_t ;
+} metalink_context ;
 
-static void _free_mirror(wget_metalink_mirror_t *mirror)
+static void mirror_free(void *mirror)
 {
-	wget_iri_free(&mirror->iri);
+	wget_metalink_mirror *m = mirror;
+
+	if (m) {
+		wget_iri_free((wget_iri **) &m->iri);
+		xfree(m);
+	}
 }
 
-static void _add_piece(_metalink_context_t *ctx, const char *value)
+static void add_piece(metalink_context *ctx, const char *value)
 {
-	wget_metalink_t *metalink = ctx->metalink;
+	wget_metalink *metalink = ctx->metalink;
 
 	sscanf(value, "%127s", ctx->hash);
 
 	if (ctx->length && *ctx->hash_type && *ctx->hash) {
 		// hash for a piece of the file
-		wget_metalink_piece_t piece, *piecep;
+		wget_metalink_piece piece, *piecep;
 
 		if (!metalink->pieces)
 			metalink->pieces = wget_vector_create(32, NULL);
@@ -92,78 +96,95 @@ static void _add_piece(_metalink_context_t *ctx, const char *value)
 				piece.position = 0; // integer overflow
 		} else
 			piece.position = 0;
-		wget_vector_add(metalink->pieces, &piece, sizeof(wget_metalink_piece_t));
+		wget_vector_add_memdup(metalink->pieces, &piece, sizeof(wget_metalink_piece));
 	}
 
 	*ctx->hash = 0;
 }
 
-static void _add_file_hash(_metalink_context_t *ctx, const char *value)
+static void add_file_hash(metalink_context *ctx, const char *value)
 {
-	wget_metalink_t *metalink = ctx->metalink;
+	wget_metalink *metalink = ctx->metalink;
 
 	sscanf(value, "%127s", ctx->hash);
 
 	if (*ctx->hash_type && *ctx->hash) {
 		// hashes for the complete file
-		wget_metalink_hash_t hash;
+		wget_metalink_hash hash;
 
-		memset(&hash, 0, sizeof(wget_metalink_hash_t));
+		memset(&hash, 0, sizeof(wget_metalink_hash));
 		wget_strscpy(hash.type, ctx->hash_type, sizeof(hash.type));
 		wget_strscpy(hash.hash_hex, ctx->hash, sizeof(hash.hash_hex));
 
 		if (!metalink->hashes)
 			metalink->hashes = wget_vector_create(4, NULL);
-		wget_vector_add(metalink->hashes, &hash, sizeof(wget_metalink_hash_t));
+		wget_vector_add_memdup(metalink->hashes, &hash, sizeof(wget_metalink_hash));
 	}
 
 	*ctx->hash_type = *ctx->hash = 0;
 }
 
-static void _add_mirror(_metalink_context_t *ctx, const char *value)
+static void add_mirror(metalink_context *ctx, const char *value)
 {
-	while (c_isspace(*value))
-		value++;
+	wget_iri *iri = wget_iri_parse(value, NULL);
 
-	if (wget_strncasecmp_ascii(value, "http:", 5) && wget_strncasecmp_ascii(value, "https:", 6))
+	if (!iri)
 		return;
 
-	wget_metalink_t *metalink = ctx->metalink;
-	wget_metalink_mirror_t mirror;
-
-	memset(&mirror, 0, sizeof(wget_metalink_mirror_t));
-	wget_strscpy(mirror.location, ctx->location, sizeof(mirror.location));
-	mirror.priority = ctx->priority;
-	mirror.iri = wget_iri_parse(value, NULL);
-
-	if (!mirror.iri)
+	if (!wget_iri_supported(iri)) {
+		error_printf(_("Mirror scheme not supported: '%s'\n"), value);
+		wget_iri_free(&iri);
 		return;
+	}
+
+/*	if (iri->scheme == WGET_IRI_SCHEME_HTTP)
+		test_modify_hsts(iri);
+
+	if (config.https_only && iri->scheme != WGET_IRI_SCHEME_HTTPS) {
+		info_printf(_("Mirror '%s' dropped (https-only requested)\n"), value);
+		wget_iri_free(&iri);
+		return;
+	}
+
+	if (iri->scheme == WGET_IRI_SCHEME_HTTP && config.https_enforce) {
+		wget_iri_set_scheme(iri, WGET_IRI_SCHEME_HTTPS);
+	}
+*/
+
+	wget_metalink *metalink = ctx->metalink;
+	wget_metalink_mirror *mirror = wget_calloc(1, sizeof(wget_metalink_mirror));
+
+	wget_strscpy(mirror->location, ctx->location, sizeof(mirror->location));
+	mirror->priority = ctx->priority;
+	mirror->iri = iri;
 
 	if (!metalink->mirrors) {
 		metalink->mirrors = wget_vector_create(4, NULL);
-		wget_vector_set_destructor(metalink->mirrors, (wget_vector_destructor_t)_free_mirror);
+		wget_vector_set_destructor(metalink->mirrors, mirror_free);
 	}
-	wget_vector_add(metalink->mirrors, &mirror, sizeof(wget_metalink_mirror_t));
+	wget_vector_add(metalink->mirrors, mirror);
 
 	*ctx->location = 0;
 	ctx->priority = 999999;
 }
 
-static void _metalink_parse(void *context, int flags, const char *dir, const char *attr, const char *val, size_t len, size_t pos G_GNUC_WGET_UNUSED)
+static void metalink_parse(void *context, int flags, const char *dir, const char *attr, const char *val, size_t len, size_t pos WGET_GCC_UNUSED)
 {
-	_metalink_context_t *ctx = context;
-	char value[len + 1];
+	metalink_context *ctx = context;
+	char valuebuf[1024];
+	const char *value;
 
 	// info_printf("\n%02X %s %s '%s'\n", flags, dir, attr, value);
-	if (!(flags & (XML_FLG_CONTENT | XML_FLG_ATTRIBUTE))) return; // ignore comments
+	if (!(flags & (XML_FLG_CONTENT | XML_FLG_ATTRIBUTE)))
+		return; // ignore comments
 
-	if (wget_strncasecmp_ascii(dir, "/metalink/file", 14)) return;
+	if (wget_strncasecmp_ascii(dir, "/metalink/file", 14))
+		return;
 
 	dir += 14;
 
-	if (val)
-			memcpy(value, val, len);
-	value[len] = 0;
+	if (!(value = wget_strmemcpy_a(valuebuf, sizeof(valuebuf), val ? val : "", len)))
+		return;
 
 	if (!wget_strncasecmp_ascii(dir, "s/file", 6)) {
 		// metalink 3 XML format
@@ -203,13 +224,13 @@ static void _metalink_parse(void *context, int flags, const char *dir, const cha
 			}
 		} else {
 			if (!wget_strcasecmp_ascii(dir, "/verification/pieces/hash")) {
-				_add_piece(ctx, value);
+				add_piece(ctx, value);
 			} else if (!wget_strcasecmp_ascii(dir, "/verification/hash")) {
-				_add_file_hash(ctx, value);
+				add_file_hash(ctx, value);
 			} else if (!wget_strcasecmp_ascii(dir, "/size")) {
 				ctx->metalink->size = atoll(value);
 			} else if (!wget_strcasecmp_ascii(dir, "/resources/url")) {
-				_add_mirror(ctx, value);
+				add_mirror(ctx, value);
 			}
 		}
 	} else {
@@ -240,27 +261,30 @@ static void _metalink_parse(void *context, int flags, const char *dir, const cha
 			}
 		} else {
 			if (!wget_strcasecmp_ascii(dir, "/pieces/hash")) {
-				_add_piece(ctx, value);
+				add_piece(ctx, value);
 			} else if (!wget_strcasecmp_ascii(dir, "/hash")) {
-				_add_file_hash(ctx, value);
+				add_file_hash(ctx, value);
 			} else if (!wget_strcasecmp_ascii(dir, "/size")) {
 				ctx->metalink->size = atoll(value);
 			} else if (!wget_strcasecmp_ascii(dir, "/url")) {
-				_add_mirror(ctx, value);
+				add_mirror(ctx, value);
 			}
 		}
 	}
+
+	if (value != valuebuf)
+		xfree(value);
 }
 
-wget_metalink_t *wget_metalink_parse(const char *xml)
+wget_metalink *wget_metalink_parse(const char *xml)
 {
 	if (!xml)
 		return NULL;
 
-	wget_metalink_t *metalink = xcalloc(1, sizeof(wget_metalink_t));
-	_metalink_context_t ctx = { .metalink = metalink, .priority = 999999, .location = "-" };
+	wget_metalink *metalink = wget_calloc(1, sizeof(wget_metalink));
+	metalink_context ctx = { .metalink = metalink, .priority = 999999, .location = "-" };
 
-	if (wget_xml_parse_buffer(xml, _metalink_parse, &ctx, 0) != WGET_E_SUCCESS) {
+	if (wget_xml_parse_buffer(xml, metalink_parse, &ctx, 0) != WGET_E_SUCCESS) {
 		error_printf(_("Error in parsing XML"));
 		wget_metalink_free(&metalink);
 	}
@@ -268,7 +292,7 @@ wget_metalink_t *wget_metalink_parse(const char *xml)
 	return metalink;
 }
 
-void wget_metalink_free(wget_metalink_t **metalink)
+void wget_metalink_free(wget_metalink **metalink)
 {
 	if (metalink && *metalink) {
 		xfree((*metalink)->name);
@@ -279,15 +303,16 @@ void wget_metalink_free(wget_metalink_t **metalink)
 	}
 }
 
-static int G_GNUC_WGET_PURE _compare_mirror(wget_metalink_mirror_t **m1, wget_metalink_mirror_t **m2)
+WGET_GCC_PURE
+static int compare_mirror(wget_metalink_mirror **m1, wget_metalink_mirror **m2)
 {
 	return (*m1)->priority - (*m2)->priority;
 }
 
-void wget_metalink_sort_mirrors(wget_metalink_t *metalink)
+void wget_metalink_sort_mirrors(wget_metalink *metalink)
 {
 	if (metalink) {
-		wget_vector_setcmpfunc(metalink->mirrors, (wget_vector_compare_t)_compare_mirror);
+		wget_vector_setcmpfunc(metalink->mirrors, (wget_vector_compare_fn *) compare_mirror);
 		wget_vector_sort(metalink->mirrors);
 	}
 }
