@@ -1,6 +1,6 @@
 /*
- * Copyright(c) 2012 Tim Ruehsen
- * Copyright(c) 2016-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2012 Tim Ruehsen
+ * Copyright (c) 2016-2022 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -50,10 +50,10 @@
 static int seeded;
 static char statebuf[64];
 static struct random_data state;
-static wget_thread_mutex_t mutex;
+static wget_thread_mutex mutex;
 static bool initialized;
 
-static void __attribute__ ((constructor)) _wget_random_init(void)
+static void __attribute__ ((constructor)) random_init(void)
 {
 	if (!initialized) {
 		wget_thread_mutex_init(&mutex);
@@ -61,7 +61,7 @@ static void __attribute__ ((constructor)) _wget_random_init(void)
 	}
 }
 
-static void __attribute__ ((destructor)) _wget_random_exit(void)
+static void __attribute__ ((destructor)) random_exit(void)
 {
 	if (initialized) {
 		wget_thread_mutex_destroy(&mutex);
@@ -73,26 +73,26 @@ static void __attribute__ ((destructor)) _wget_random_exit(void)
  * Random API initialization, allocating/preparing the internal resources.
  *
  * On systems with automatic library constructors, this function
- * doesn't have to be called explictly.
+ * doesn't have to be called explicitly.
  *
  * This function is not thread-safe.
  */
 void wget_random_init(void)
 {
-	_wget_random_init();
+	random_init();
 }
 
 /**
  * Random API deinitialization, free'ing all internal resources.
  *
  * On systems with automatic library destructors, this function
- * doesn't have to be called explictly.
+ * doesn't have to be called explicitly.
  *
  * This function is not thread-safe.
  */
 void wget_random_exit(void)
 {
-	_wget_random_exit();
+	random_exit();
 }
 
 /**
