@@ -1,5 +1,5 @@
 /*
- * Copyright(c) 2015-2018 Free Software Foundation, Inc.
+ * Copyright (c) 2015-2022 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -17,22 +17,46 @@
  * along with libwget.  If not, see <https://www.gnu.org/licenses/>.
  *
  *
+ * SSL/TLS routines, with OpenSSL as the backend engine
+ *
+ * Author: Ander Juaristi
  */
 
 #include <config.h>
 
 #include <dirent.h>
+#include <limits.h> // INT_MAX
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 #include <openssl/ocsp.h>
 #include <openssl/crypto.h>
 #include <openssl/ossl_typ.h>
+#include <openssl/x509.h>
 #include <openssl/x509_vfy.h>
 #include <openssl/x509v3.h>
+#include <openssl/asn1.h>
+
+#ifdef _WIN32
+#  include <w32sock.h>
+#else
+#  define FD_TO_SOCKET(x) (x)
+#  define SOCKET_TO_FD(x) (x)
+#endif
+
+#ifdef LIBRESSL_VERSION_NUMBER
+  #ifndef TLS_MAX_VERSION
+    #ifndef TLS1_3_VERSION
+      #define TLS1_3_VERSION TLS1_2_VERSION
+      #define TLS_MAX_VERSION TLS1_2_VERSION
+    #else
+      #define TLS_MAX_VERSION TLS1_3_VERSION
+    #endif
+  #endif
+#endif
 
 #include <wget.h>
-#include "net.h"
 #include "private.h"
+#include "net.h"
 
 static wget_tls_stats_callback
 	*tls_stats_callback;
@@ -44,7 +68,7 @@ static wget_ocsp_stats_callback
 static void
 	*ocsp_stats_ctx;
 
-static struct _config
+static struct config
 {
 	const char
 		*secure_protocol,
@@ -71,11 +95,13 @@ static struct _config
 		check_hostname :1,
 		print_info :1,
 		ocsp :1,
-		ocsp_stapling :1;
-} _config = {
+		ocsp_date :1,
+		ocsp_stapling :1,
+		ocsp_nonce :1;
+} config = {
 	.check_certificate = 1,
 	.check_hostname = 1,
-#ifdef HAVE_GNUTLS_OCSP_H
+#ifdef WITH_OCSP
 	.ocsp = 1,
 	.ocsp_stapling = 1,
 #endif
@@ -89,30 +115,31 @@ static struct _config
 #endif
 	};
 
-static int _init;
-static __thread int _ex_data_idx;
-static __thread CRYPTO_EX_DATA _crypto_ex_data;
-static wget_thread_mutex _mutex;
+static int init;
+static wget_thread_mutex mutex;
 
 static SSL_CTX *_ctx;
+static int store_userdata_idx;
 
 /*
  * Constructor & destructor
  */
-static void __attribute__ ((constructor)) _wget_tls_init(void)
+static void __attribute__ ((constructor)) tls_init(void)
 {
-	if (!_mutex)
-		wget_thread_mutex_init(&_mutex);
-	if (!_ex_data_idx)
-		_ex_data_idx = -1;
+	if (!mutex)
+		wget_thread_mutex_init(&mutex);
+
+	store_userdata_idx = X509_STORE_CTX_get_ex_new_index(
+			0, NULL,	/* argl, argp */
+			NULL,		/* new_func */
+			NULL,		/* dup_func */
+			NULL);		/* free_func */
 }
 
-static void __attribute__ ((destructor)) _wget_tls_exit(void)
+static void __attribute__ ((destructor)) tls_exit(void)
 {
-	if (_mutex)
-		wget_thread_mutex_destroy(&_mutex);
-	if (_ex_data_idx)
-		_ex_data_idx = -1;
+	if (mutex)
+		wget_thread_mutex_destroy(&mutex);
 }
 
 /*
@@ -177,28 +204,28 @@ void wget_ssl_set_config_string(int key, const char *value)
 {
 	switch (key) {
 	case WGET_SSL_SECURE_PROTOCOL:
-		_config.secure_protocol = value;
+		config.secure_protocol = value;
 		break;
 	case WGET_SSL_CA_DIRECTORY:
-		_config.ca_directory = value;
+		config.ca_directory = value;
 		break;
 	case WGET_SSL_CA_FILE:
-		_config.ca_file = value;
+		config.ca_file = value;
 		break;
 	case WGET_SSL_CERT_FILE:
-		_config.cert_file = value;
+		config.cert_file = value;
 		break;
 	case WGET_SSL_KEY_FILE:
-		_config.key_file = value;
+		config.key_file = value;
 		break;
 	case WGET_SSL_CRL_FILE:
-		_config.crl_file = value;
+		config.crl_file = value;
 		break;
 	case WGET_SSL_OCSP_SERVER:
-		_config.ocsp_server = value;
+		config.ocsp_server = value;
 		break;
 	case WGET_SSL_ALPN:
-		_config.alpn = value;
+		config.alpn = value;
 		break;
 	default:
 		error_printf(_("Unknown configuration key %d (maybe this config value should be of another type?)\n"), key);
@@ -232,13 +259,13 @@ void wget_ssl_set_config_object(int key, void *value)
 {
 	switch (key) {
 	case WGET_SSL_OCSP_CACHE:
-		_config.ocsp_cert_cache = (wget_ocsp_db *) value;
+		config.ocsp_cert_cache = (wget_ocsp_db *) value;
 		break;
 	case WGET_SSL_SESSION_CACHE:
-		_config.tls_session_cache = (wget_tls_session_db *) value;
+		config.tls_session_cache = (wget_tls_session_db *) value;
 		break;
 	case WGET_SSL_HPKP_CACHE:
-		_config.hpkp_cache = (wget_hpkp_db *) value;
+		config.hpkp_cache = (wget_hpkp_db *) value;
 		break;
 	default:
 		error_printf(_("Unknown configuration key %d (maybe this config value should be of another type?)\n"), key);
@@ -275,33 +302,44 @@ void wget_ssl_set_config_object(int key, void *value)
  *
  *  - WGET_SSL_OCSP: whether or not OCSP should be used. The default is yes (1).
  *  - WGET_SSL_OCSP_STAPLING: whether or not OCSP stapling should be used. The default is yes (1).
+ *  - WGET_SSL_OCSP_NONCE: whether or not an OCSP nonce should be sent in the request. The default is yes (1).
+ *  If a nonce was sent in the request, the OCSP verification will fail if the response nonce doesn't match.
+ *  However if the response does not include a nonce extension, verification will be allowed to continue.
+ *  The OCSP nonce extension is not a critical one.
+ *  - WGET_SSL_OCSP_DATE: Reject the OCSP response if it's older than 3 days.
  */
 void wget_ssl_set_config_int(int key, int value)
 {
 	switch (key) {
 	case WGET_SSL_CHECK_CERTIFICATE:
-		_config.check_certificate = value;
+		config.check_certificate = value;
 		break;
 	case WGET_SSL_CHECK_HOSTNAME:
-		_config.check_hostname = value;
+		config.check_hostname = value;
 		break;
 	case WGET_SSL_PRINT_INFO:
-		_config.print_info = value;
+		config.print_info = value;
 		break;
 	case WGET_SSL_CA_TYPE:
-		_config.ca_type = (char) value;
+		config.ca_type = (char) value;
 		break;
 	case WGET_SSL_CERT_TYPE:
-		_config.cert_type = (char) value;
+		config.cert_type = (char) value;
 		break;
 	case WGET_SSL_KEY_TYPE:
-		_config.key_type = (char) value;
+		config.key_type = (char) value;
 		break;
 	case WGET_SSL_OCSP:
-		_config.ocsp = value;
+		config.ocsp = value;
 		break;
 	case WGET_SSL_OCSP_STAPLING:
-		_config.ocsp_stapling = value;
+		config.ocsp_stapling = value;
+		break;
+	case WGET_SSL_OCSP_NONCE:
+		config.ocsp_nonce = value;
+		break;
+	case WGET_SSL_OCSP_DATE:
+		config.ocsp_date = value;
 		break;
 	default:
 		error_printf(_("Unknown configuration key %d (maybe this config value should be of another type?)\n"), key);
@@ -317,7 +355,7 @@ static int openssl_load_crl(X509_STORE *store, const char *crl_file)
 
 	if (!X509_load_crl_file(lookup, crl_file, X509_FILETYPE_PEM))
 		return WGET_E_UNKNOWN;
-	if (!X509_STORE_set_flags(store, X509_V_FLAG_CRL_CHECK_ALL | X509_V_FLAG_USE_DELTAS))
+	if (!X509_STORE_set_flags(store, X509_V_FLAG_CRL_CHECK | X509_V_FLAG_CRL_CHECK_ALL | X509_V_FLAG_USE_DELTAS))
 		return WGET_E_UNKNOWN;
 
 	return 0;
@@ -440,7 +478,7 @@ end:
 	return retval;
 }
 
-static int verify_hpkp(const char *hostname, X509 *subject_cert)
+static int verify_hpkp(const char *hostname, X509 *subject_cert, wget_hpkp_stats_result *hpkp_stats)
 {
 	int retval, spki_len;
 	unsigned char *spki = NULL;
@@ -451,27 +489,29 @@ static int verify_hpkp(const char *hostname, X509 *subject_cert)
 		return -1;
 
 	/* Lookup database */
-	retval = wget_hpkp_db_check_pubkey(_config.hpkp_cache,
+	retval = wget_hpkp_db_check_pubkey(config.hpkp_cache,
 		hostname,
 		spki, spki_len);
 
-	/* TODO update stats here */
-
 	switch (retval) {
 	case 1:
 		debug_printf("Matching HPKP pinning found for host '%s'\n", hostname);
+		*hpkp_stats = WGET_STATS_HPKP_MATCH;
 		retval = 0;
 		break;
 	case 0:
 		debug_printf("No HPKP pinning found for host '%s'\n", hostname);
+		*hpkp_stats = WGET_STATS_HPKP_NO;
 		retval = 1;
 		break;
 	case -2:
 		debug_printf("Public key for host '%s' does not match\n", hostname);
+		*hpkp_stats = WGET_STATS_HPKP_NOMATCH;
 		retval = -1;
 		break;
 	default:
 		debug_printf("Could not check HPKP pinning for host '%s' (%d)\n", hostname, retval);
+		*hpkp_stats = WGET_STATS_HPKP_ERROR;
 		retval = 0;
 	}
 
@@ -479,48 +519,662 @@ static int verify_hpkp(const char *hostname, X509 *subject_cert)
 	return retval;
 }
 
-/*
- * This is our custom revocation check function.
- * It will be invoked by OpenSSL at some point during the TLS handshake.
- * It takes the server's certificate chain, and its purpose is to check the revocation
- * status for each certificate in it. We validate certs against HPKP and OCSP here.
- * This function should return 1 on success (the whole cert chain is valid) and 0 on failure.
- */
-static int _openssl_revocation_check_fn(X509_STORE_CTX *storectx)
+static int check_cert_chain_for_hpkp(STACK_OF(X509) *certs, const char *hostname, wget_hpkp_stats_result *hpkp_stats)
 {
-	int pin_ok = 0, retval;
-	X509 *cert = NULL;
-	const char *hostname;
-	STACK_OF(X509) *certs = X509_STORE_CTX_get1_chain(storectx);
+	int retval, pin_ok = 0;
+	X509 *cert;
 	unsigned cert_list_size = sk_X509_num(certs);
 
-	hostname = CRYPTO_get_ex_data(&_crypto_ex_data, _ex_data_idx);
-
-	/* Check the whole cert chain against HPKP database */
-	if (!_config.hpkp_cache)
-		return 1;
-
 	for (unsigned i = 0; i < cert_list_size; i++) {
 		cert = sk_X509_value(certs, i);
 
-		if ((retval = verify_hpkp(hostname, cert)) >= 0)
+		if ((retval = verify_hpkp(hostname, cert, hpkp_stats)) >= 0)
 			pin_ok = 1;
 		if (retval == 1)
 			break;
 	}
 
-	if (!pin_ok)
-		error_printf(_("Public key pinning mismatch.\n"));
-
 	return pin_ok;
 }
 
+struct verification_flags {
+	SSL
+		*ssl;
+	const char
+		*hostname;
+	X509_STORE
+		*certstore;
+	unsigned int
+		cert_chain_size;
+	wget_hpkp_stats_result
+		hpkp_stats;
+	bool
+		verifying_ocsp,
+		ocsp_checked;
+};
+
+static int check_ocsp_response(OCSP_RESPONSE *,
+		STACK_OF(X509) *,
+		X509_STORE *,
+		bool);
+
+static int ocsp_resp_cb(SSL *s, void *arg)
+{
+	int result;
+	long ocsp_resp_len;
+	const unsigned char *ocsp_resp_raw = NULL;
+	OCSP_RESPONSE *ocspresp;
+	STACK_OF(X509) *certstack;
+	struct verification_flags *vflags = arg;
+
+	if (!vflags)
+		return 0;
+
+	ocsp_resp_len = SSL_get_tlsext_status_ocsp_resp(s, &ocsp_resp_raw);
+	if (ocsp_resp_len == -1) {
+		debug_printf("No stapled OCSP response was received. Continuing.\n");
+		return 1;
+	}
+
+	ocspresp = d2i_OCSP_RESPONSE(NULL, &ocsp_resp_raw, ocsp_resp_len);
+	if (!ocspresp) {
+		error_printf(_("Got a stapled OCSP response, but could not parse it. Aborting.\n"));
+		return 0;
+	}
+
+	certstack = SSL_get_peer_cert_chain(vflags->ssl);
+	if (!certstack) {
+		error_printf(_("Could not get server's cert stack\n"));
+		return 0;
+	}
+
+	result = check_ocsp_response(ocspresp,
+		certstack,
+		vflags->certstore,
+		0);
+
+	if (result == -1) {
+		OCSP_RESPONSE_free(ocspresp);
+		error_printf(_("Could not verify stapled OCSP response. Aborting.\n"));
+		return 0;
+	}
+
+	OCSP_RESPONSE_free(ocspresp);
+	debug_printf("Got a stapled OCSP response. Length: %ld. Status: OK\n", ocsp_resp_len);
+	return 1;
+}
+
+static OCSP_REQUEST *send_ocsp_request(const char *uri,
+		OCSP_CERTID *certid,
+		wget_http_response **response)
+{
+	OCSP_REQUEST *ocspreq;
+	wget_http_response *resp;
+	unsigned char *ocspreq_bytes = NULL;
+	size_t ocspreq_bytes_len;
+
+	ocspreq = OCSP_REQUEST_new();
+	if (!ocspreq)
+		goto end;
+
+	if (!OCSP_request_add0_id(ocspreq, certid)) {
+		OCSP_REQUEST_free(ocspreq);
+		ocspreq = NULL;
+		goto end;
+	}
+
+	if (config.ocsp_nonce && !OCSP_request_add1_nonce(ocspreq, NULL, 0)) {
+		OCSP_REQUEST_free(ocspreq);
+		ocspreq = NULL;
+		goto end;
+	}
+
+	ocspreq_bytes_len = i2d_OCSP_REQUEST(ocspreq, &ocspreq_bytes);
+	if (!ocspreq_bytes || !ocspreq_bytes_len) {
+		OCSP_REQUEST_free(ocspreq);
+		ocspreq = NULL;
+		goto end;
+	}
+
+	resp = wget_http_get(
+		WGET_HTTP_URL, uri,
+		WGET_HTTP_SCHEME, "POST",
+		WGET_HTTP_HEADER_ADD, "Accept-Encoding", "identity",
+		WGET_HTTP_HEADER_ADD, "Accept", "*/*",
+		WGET_HTTP_HEADER_ADD, "Content-Type", "application/ocsp-request",
+		WGET_HTTP_MAX_REDIRECTIONS, 5,
+		WGET_HTTP_BODY, ocspreq_bytes, ocspreq_bytes_len,
+		0);
+
+	OPENSSL_free(ocspreq_bytes);
+
+	if (resp) {
+		*response = resp;
+	} else {
+		OCSP_REQUEST_free(ocspreq);
+		ocspreq = NULL;
+	}
+
+end:
+	return ocspreq;
+}
+
+static const char *get_printable_ocsp_reason_desc(int reason)
+{
+	switch (reason) {
+	case OCSP_REVOKED_STATUS_NOSTATUS:
+		return "not given";
+	case OCSP_REVOKED_STATUS_UNSPECIFIED:
+		return "unspecified";
+	case OCSP_REVOKED_STATUS_KEYCOMPROMISE:
+		return "key compromise";
+	case OCSP_REVOKED_STATUS_CACOMPROMISE:
+		return "CA compromise";
+	case OCSP_REVOKED_STATUS_AFFILIATIONCHANGED:
+		return "affiliation changed";
+	case OCSP_REVOKED_STATUS_SUPERSEDED:
+		return "superseded";
+	case OCSP_REVOKED_STATUS_CESSATIONOFOPERATION:
+		return "cessation of operation";
+	case OCSP_REVOKED_STATUS_CERTIFICATEHOLD:
+		return "certificate hold";
+	case OCSP_REVOKED_STATUS_REMOVEFROMCRL:
+		return "remove from CRL";
+	default:
+		return "unknown reason";
+	}
+}
+
+static void print_ocsp_response_status(int status)
+{
+	char msg[64];
+	const char *status_string;
+
+	switch (status) {
+	case OCSP_RESPONSE_STATUS_SUCCESSFUL:
+		status_string = "successful";
+		break;
+	case OCSP_RESPONSE_STATUS_MALFORMEDREQUEST:
+		status_string = "malformed request";
+		break;
+	case OCSP_RESPONSE_STATUS_INTERNALERROR:
+		status_string = "internal error";
+		break;
+	case OCSP_RESPONSE_STATUS_TRYLATER:
+		status_string = "try later";
+		break;
+	case OCSP_RESPONSE_STATUS_SIGREQUIRED:
+		status_string = "signature required";
+		break;
+	case OCSP_RESPONSE_STATUS_UNAUTHORIZED:
+		status_string = "unauthorized";
+		break;
+	default:
+		wget_snprintf(msg, sizeof(msg), "unknown status code %d", status);
+		status_string = msg;
+		break;
+	}
+
+	debug_printf("*** OCSP response status: %s\n", status_string);
+}
+
+static void print_ocsp_cert_status(int status, int reason)
+{
+	char msg[64];
+	const char *reason_string;
+
+	switch (status) {
+	case V_OCSP_CERTSTATUS_GOOD:
+		reason_string = "good";
+		break;
+	case V_OCSP_CERTSTATUS_UNKNOWN:
+		reason_string = "unknown";
+		break;
+	case V_OCSP_CERTSTATUS_REVOKED:
+		wget_snprintf(msg, sizeof(msg), "revoked (%s)", get_printable_ocsp_reason_desc(reason));
+		reason_string = msg;
+		break;
+	default:
+		reason_string = "invalid status code";
+		break;
+	}
+
+	debug_printf("*** OCSP cert status: %s\n", reason_string);
+}
+
+static void print_openssl_time(const char *prefix, const ASN1_GENERALIZEDTIME *t)
+{
+	int nread;
+	char buf[128];
+	BIO *mem = BIO_new(BIO_s_mem());
+
+	ASN1_GENERALIZEDTIME_print(mem, t);
+
+	nread = BIO_read(mem, buf, sizeof(buf)-1);
+	if (nread > 0) {
+		buf[nread] = '\0';
+		debug_printf("%s%s\n", prefix, buf);
+	} else {
+		error_printf(_("ERROR: print_openssl_time: BIO_read failed\n"));
+	}
+
+	BIO_free_all(mem);
+}
+
+static int check_ocsp_response_times(const ASN1_GENERALIZEDTIME *thisupd,
+				     const ASN1_GENERALIZEDTIME *nextupd)
+{
+	int day, sec, retval = -1;
+	ASN1_TIME *now;
+
+	now = ASN1_TIME_adj(NULL, time(NULL), 0, 0);
+	if (!now) {
+		error_printf(_("Could not get current time!\n"));
+		return -1;
+	}
+
+	print_openssl_time("*** OCSP issued time: ", thisupd);
+
+	if (!nextupd) {
+		debug_printf("OCSP nextUpd not set. Checking thisUpd is not too old.\n");
+		if (!ASN1_TIME_diff(&day, &sec, now, thisupd)) {
+			error_printf(_("Could not compute time difference for thisUpd. Aborting.\n"));
+			goto end;
+		}
+		if (day < -3) {
+			error_printf(_("*** OCSP response thisUpd is too old. Aborting.\n"));
+			goto end;
+		}
+
+		retval = 0;
+		goto end;
+	}
+
+	print_openssl_time("*** OCSP update time: ", nextupd);
+
+	if (!ASN1_TIME_diff(&day, &sec, now, nextupd)) {
+		error_printf(_("Could not compute time difference for nextUpd. Aborting.\n"));
+		goto end;
+	}
+
+	if (day < 0 || (day == 0 && sec < 0)) {
+		error_printf(_("*** OCSP next update is in the past!\n"));
+		goto end;
+	}
+
+	retval = 0;
+
+end:
+	ASN1_STRING_free(now);
+	return retval;
+}
+
+static int check_ocsp_response(OCSP_RESPONSE *ocspresp,
+		STACK_OF(X509) *certstack,
+		X509_STORE *certstore,
+		bool check_time)
+{
+	int
+		retval = -1,
+		status, reason;
+	OCSP_BASICRESP *ocspbs = NULL;
+	OCSP_SINGLERESP *single;
+	ASN1_GENERALIZEDTIME *revtime = NULL,
+			*thisupd = NULL,
+			*nextupd = NULL;
+
+	status = OCSP_response_status(ocspresp);
+	print_ocsp_response_status(status);
+
+	if (status != OCSP_RESPONSE_STATUS_SUCCESSFUL) {
+		error_printf(_("Unsuccessful OCSP response\n"));
+		goto end;
+	}
+
+	if (!(ocspbs = OCSP_response_get1_basic(ocspresp)))
+		goto end;
+
+	if (!OCSP_basic_verify(ocspbs, certstack, certstore, 0)) {
+		error_printf(_("Could not verify OCSP certificate chain\n"));
+		goto end;
+	}
+
+	single = OCSP_resp_get0(ocspbs, 0);
+	if (!single) {
+		error_printf(_("Could not parse OCSP single response\n"));
+		goto end;
+	}
+
+	// thisupd and nextupd are internal pointers and MUST NOT be freed
+	status = OCSP_single_get0_status(single, &reason, &revtime, &thisupd, &nextupd);
+	if (status == -1) {
+		error_printf(_("Could not obtain OCSP response status\n"));
+		goto end;
+	}
+
+	print_ocsp_cert_status(status, reason);
+
+	if (status == V_OCSP_CERTSTATUS_REVOKED) {
+		print_openssl_time("*** Certificate revoked by OCSP at: ", revtime);
+		retval = 1; // Failure
+		goto end;
+	}
+
+	/* Check time is within an acceptable range */
+	if (check_time) {
+		if (!thisupd) {
+			error_printf(_("Could not get 'thisUpd' from OCSP response. Cannot check time.\n"));
+			goto end;
+		}
+
+		if (check_ocsp_response_times(thisupd, nextupd) < 0) {
+			retval = 1; // Failure
+			goto end;
+		}
+	}
+
+	retval = 0; // Success!
+
+end:
+	if (ocspbs)
+		OCSP_BASICRESP_free(ocspbs);
+	return retval;
+}
+
+static int verify_ocsp(const char *ocsp_uri,
+		X509 *subject_cert, X509 *issuer_cert,
+		STACK_OF(X509) *certs, X509_STORE *certstore,
+		bool check_time, bool check_nonce)
+{
+	int retval;
+	wget_http_response *resp;
+	const unsigned char *body;
+	OCSP_CERTID *certid;
+	OCSP_REQUEST *ocspreq;
+	OCSP_RESPONSE *ocspresp;
+	OCSP_BASICRESP *ocspbs = NULL;
+
+	/* Generate CertID and OCSP request */
+	certid = OCSP_cert_to_id(EVP_sha1(), subject_cert, issuer_cert);
+	if (!(ocspreq = send_ocsp_request(ocsp_uri,
+			certid,
+			&resp)))
+		return -1;
+
+	/* Check response */
+	body = (const unsigned char *) resp->body->data;
+	ocspresp = d2i_OCSP_RESPONSE(NULL, &body, resp->body->length);
+	if (!ocspresp) {
+		wget_http_free_response(&resp);
+		OCSP_REQUEST_free(ocspreq);
+		return -1;
+	}
+
+	if ((retval = check_ocsp_response(ocspresp, certs, certstore, check_time)) != 0)
+		goto end;
+
+	if (check_nonce) {
+		if (!(ocspbs = OCSP_response_get1_basic(ocspresp))) {
+			error_printf(_("Could not obtain OCSP_BASICRESPONSE\n"));
+			retval = -1;
+			goto end;
+		}
+
+		if (!OCSP_check_nonce(ocspreq, ocspbs)) {
+			error_printf(_("OCSP nonce does not match\n"));
+			retval = 1; // Failure
+			goto end;
+		}
+
+		OCSP_BASICRESP_free(ocspbs);
+		ocspbs = NULL;
+	}
+
+	retval = 0; // Success
+
+end:
+	if (ocspbs)
+		OCSP_BASICRESP_free(ocspbs);
+	wget_http_free_response(&resp);
+	OCSP_RESPONSE_free(ocspresp);
+	OCSP_REQUEST_free(ocspreq);
+	return retval;
+}
+
+static char *read_ocsp_uri_from_certificate(X509 *cert)
+{
+	STACK_OF(OPENSSL_STRING) *str_stack = X509_get1_ocsp(cert);
+
+	if (str_stack && sk_OPENSSL_STRING_num(str_stack) > 0) {
+		char *uri = wget_strdup(sk_OPENSSL_STRING_value(str_stack, 0));
+		X509_email_free(str_stack); // utterly misnamed, it simply frees a stack of strings.
+		return uri;
+	}
+
+	return NULL;
+}
+
+static char *compute_cert_fingerprint(X509 *cert)
+{
+	/*
+	 * OpenSSL does not provide a function that calculates the cert fingerprint directly
+	 * (like GnuTLS' gnutls_x509_crt_get_fingerprint()), but we can code it away. Fingerprint
+	 * is basically a SHA-256 hash of the DER-encoded certificate.
+	 */
+	EVP_MD_CTX *mdctx = EVP_MD_CTX_new();
+	char *hexstring = NULL;
+	unsigned char
+		*der_output = NULL,
+		*digest_output = NULL;
+	int
+		der_length,
+		digest_length,
+		hexstring_length;
+
+	if ((der_length = i2d_X509(cert, &der_output)) < 0)
+		goto bail;
+
+	/* Compute SHA-256 digest of the DER-encoded certificate */
+	digest_length = EVP_MD_size(EVP_sha256());
+	digest_output = wget_malloc(digest_length);
+	if (!digest_output)
+		goto bail;
+
+	if (!EVP_DigestInit_ex(mdctx, EVP_sha256(), NULL))
+		goto bail;
+	if (!EVP_DigestUpdate(mdctx, der_output, der_length))
+		goto bail;
+	if (!EVP_DigestFinal_ex(mdctx, digest_output, NULL))
+		goto bail;
+
+	OPENSSL_free(der_output);
+	der_output = NULL;
+
+	EVP_MD_CTX_free(mdctx);
+	mdctx = NULL;
+
+	/* Convert SHA-256 digest to hex string */
+	hexstring_length = (digest_length * 2) + 1;
+	hexstring = wget_malloc(hexstring_length);
+	if (!hexstring)
+		goto bail;
+
+	wget_memtohex(digest_output, digest_length, hexstring, hexstring_length);
+	xfree(digest_output);
+	return hexstring;
+
+bail:
+	xfree(hexstring);
+	xfree(digest_output);
+	if (der_output)
+		OPENSSL_free(der_output);
+	if (mdctx)
+		EVP_MD_CTX_free(mdctx);
+	return NULL;
+}
+
+static int check_cert_chain_for_ocsp(STACK_OF(X509) *certs, X509_STORE *store, const char *hostname)
+{
+	wget_ocsp_stats_data stats;
+	int num_ok = 0, num_revoked = 0, num_ignored = 0, revoked, ocsp_ok;
+	char
+		*ocsp_uri = NULL,
+		*fingerprint;
+	X509 *cert, *issuer_cert;
+	unsigned cert_list_size = sk_X509_num(certs);
+
+	for (unsigned i = 0; i < cert_list_size; i++) {
+		cert = sk_X509_value(certs, i);
+		issuer_cert = sk_X509_value(certs, i+1);
+
+		if (!issuer_cert)
+			break;
+
+		/* Compute cert fingerprint */
+		fingerprint = compute_cert_fingerprint(cert);
+		if (!fingerprint) {
+			error_printf(_("Could not compute certificate fingerprint for cert %u\n"), i);
+			return 0; /* Treat this as an error */
+		}
+
+		/* Check if there's already an OCSP response stored in cache */
+		if (config.ocsp_cert_cache) {
+			if (wget_ocsp_fingerprint_in_cache(config.ocsp_cert_cache, fingerprint, &revoked)) {
+				/* Found cert's fingerprint in cache */
+				if (revoked) {
+					debug_printf("Certificate %u has been revoked (cached response)\n", i);
+					num_revoked++;
+				} else {
+					debug_printf("Certificate %u is valid (cached response)\n", i);
+					num_ok++;
+				}
+
+				xfree(fingerprint);
+				continue;
+			}
+		}
+
+		if (!config.ocsp_server) {
+			ocsp_uri = read_ocsp_uri_from_certificate(cert);
+			if (!ocsp_uri) {
+				debug_printf("OCSP URI not given and not found in certificate. Skipping OCSP check for cert %u.\n",
+						i);
+				num_ignored++;
+				xfree(fingerprint);
+				continue;
+			}
+		}
+
+		debug_printf("Contacting OCSP server. URI: %s\n",
+				config.ocsp_server ? config.ocsp_server : ocsp_uri);
+
+		ocsp_ok = verify_ocsp(config.ocsp_server ? config.ocsp_server : ocsp_uri,
+				cert, issuer_cert, certs, store,
+				config.ocsp_date, config.ocsp_nonce);
+		if (ocsp_ok == 0)
+			num_ok++;
+		else if (ocsp_ok == 1)
+			num_revoked++;
+		else
+			num_ignored++;
+
+		/* Add the certificate to the OCSP cache */
+		if (ocsp_ok == 0 || ocsp_ok == 1) {
+			wget_ocsp_db_add_fingerprint(config.ocsp_cert_cache,
+				fingerprint,
+				time(NULL) + 3600, /* Cache entry valid for 1 hour */
+				(ocsp_ok == 0));   /* valid? */
+		}
+
+		xfree(fingerprint);
+		xfree(ocsp_uri);
+	}
+
+	if (ocsp_stats_callback) {
+		stats.hostname = hostname;
+		stats.nvalid = num_ok;
+		stats.nrevoked = num_revoked;
+		stats.nignored = num_ignored;
+		stats.stapling = 0;
+		ocsp_stats_callback(&stats, ocsp_stats_ctx);
+	}
+
+	return (num_revoked == 0);
+}
+
+/*
+ * This is our custom revocation check function.
+ * It will be invoked by OpenSSL at some point during the TLS handshake.
+ * It takes the server's certificate chain, and its purpose is to check the revocation
+ * status for each certificate in it. We validate certs against HPKP and OCSP here.
+ * OpenSSL will make other checks before calling this function: cert signature, CRLs, etc.
+ * This function should return the value of 'ossl_retval' on success
+ * (which retains the result of previous checks made by OpenSSL) and 0 on failure (will override
+ * OpenSSL's result, whatever it is).
+ */
+static int openssl_revocation_check_fn(int ossl_retval, X509_STORE_CTX *storectx)
+{
+	X509_STORE *store;
+	struct verification_flags *vflags;
+	STACK_OF(X509) *certs = X509_STORE_CTX_get1_chain(storectx);
+
+	if (ossl_retval == 0) {
+		/* ossl_retval == 0 means certificate was revoked by OpenSSL before entering this callback */
+		goto end;
+	}
+
+	store = X509_STORE_CTX_get0_store(storectx);
+	if (!store) {
+		error_printf(_("Could not retrieve certificate store. Will skip HPKP checks.\n"));
+		goto end;
+	}
+
+	vflags = X509_STORE_get_ex_data(store, store_userdata_idx);
+	if (!vflags) {
+		error_printf(_("Could not retrieve saved verification status flags.\n"));
+		goto end;
+	}
+
+	if (vflags->verifying_ocsp)
+		goto end;
+
+	/* Store the certificate chain size */
+	vflags->cert_chain_size = sk_X509_num(certs);
+
+	if (config.hpkp_cache) {
+		/* Check cert chain against HPKP database */
+		if (!check_cert_chain_for_hpkp(certs, vflags->hostname, &vflags->hpkp_stats)) {
+			error_printf(_("Public key pinning mismatch.\n"));
+			ossl_retval = 0;
+			goto end;
+		}
+	}
+
+	if (config.ocsp && !vflags->ocsp_checked) {
+		/* Check cert chain against OCSP */
+		vflags->verifying_ocsp = 1;
+
+		if (!check_cert_chain_for_ocsp(certs, store, vflags->hostname)) {
+			error_printf(_("Certificate revoked by OCSP.\n"));
+			ossl_retval = 0;
+			goto end;
+		}
+
+		vflags->ocsp_checked = 1;
+		vflags->verifying_ocsp = 0;
+	}
+
+end:
+	sk_X509_pop_free(certs, X509_free);
+	return ossl_retval;
+}
+
 static int openssl_init(SSL_CTX *ctx)
 {
 	int retval = 0;
 	X509_STORE *store;
 
-	if (!_config.check_certificate) {
+	if (!config.check_certificate) {
 		SSL_CTX_set_verify(ctx, SSL_VERIFY_NONE, NULL);
 		info_printf(_("Certificate check disabled. Peer's certificate will NOT be checked.\n"));
 		goto end;
@@ -533,16 +1187,16 @@ static int openssl_init(SSL_CTX *ctx)
 		goto end;
 	}
 
-	if (_config.ca_directory && *_config.ca_directory) {
-		retval = openssl_load_trust_files(ctx, _config.ca_directory);
+	if (config.ca_directory && *config.ca_directory) {
+		retval = openssl_load_trust_files(ctx, config.ca_directory);
 		if (retval < 0)
 			goto end;
 
-		if (_config.crl_file) {
+		if (config.crl_file) {
 			/* Load CRL file in PEM format. */
-			if ((retval = openssl_load_crl(store, _config.crl_file)) < 0) {
+			if ((retval = openssl_load_crl(store, config.crl_file)) < 0) {
 				error_printf(_("Could not load CRL from '%s' (%d)\n"),
-					_config.crl_file,
+					config.crl_file,
 					retval);
 				goto end;
 			}
@@ -552,16 +1206,21 @@ static int openssl_init(SSL_CTX *ctx)
 	}
 
 	/* Load individual CA file, if requested */
-	if (_config.ca_file && *_config.ca_file
-		&& !SSL_CTX_load_verify_locations(ctx, _config.ca_file, NULL))
+	if (config.ca_file && *config.ca_file
+		&& !SSL_CTX_load_verify_locations(ctx, config.ca_file, NULL))
 	{
-		error_printf(_("Could not load CA certificate from file '%s'\n"), _config.ca_file);
+		error_printf(_("Could not load CA certificate from file '%s'\n"), config.ca_file);
 	}
 
-	/* Set our custom revocation check function, for HPKP and OCSP validation */
-	X509_STORE_set_check_revocation(store, _openssl_revocation_check_fn);
+#ifdef WITH_OCSP
+	if (config.ocsp_stapling)
+		SSL_CTX_set_tlsext_status_cb(ctx, ocsp_resp_cb);
+#endif
 
-	retval = openssl_set_priorities(ctx, _config.secure_protocol);
+	/* Set our custom revocation check function, for HPKP and OCSP validation */
+	X509_STORE_set_verify_cb(store, openssl_revocation_check_fn);
+
+	retval = openssl_set_priorities(ctx, config.secure_protocol);
 
 end:
 	return retval;
@@ -594,19 +1253,23 @@ static void openssl_deinit(SSL_CTX *ctx)
  */
 void wget_ssl_init(void)
 {
-	wget_thread_mutex_lock(_mutex);
+	wget_thread_mutex_lock(mutex);
 
-	if (!_init) {
+	if (!init) {
 		_ctx = SSL_CTX_new(TLS_client_method());
 		if (_ctx && openssl_init(_ctx) == 0) {
-			_init++;
+			init++;
+#ifdef LIBRESSL_VERSION_NUMBER
+			debug_printf("LibreSSL initialized\n");
+#else
 			debug_printf("OpenSSL initialized\n");
+#endif
 		} else {
 			error_printf(_("Could not initialize OpenSSL\n"));
 		}
 	}
 
-	wget_thread_mutex_unlock(_mutex);
+	wget_thread_mutex_unlock(mutex);
 }
 
 /**
@@ -623,27 +1286,27 @@ void wget_ssl_init(void)
  */
 void wget_ssl_deinit(void)
 {
-	wget_thread_mutex_lock(_mutex);
+	wget_thread_mutex_lock(mutex);
 
-	if (_init == 1)
+	if (init == 1)
 		openssl_deinit(_ctx);
 
-	if (_init > 0)
-		_init--;
+	if (init > 0)
+		init--;
 
-	wget_thread_mutex_unlock(_mutex);
+	wget_thread_mutex_unlock(mutex);
 }
 
 static int ssl_resume_session(SSL *ssl, const char *hostname)
 {
 	void *sess = NULL;
-	unsigned long sesslen;
+	size_t sesslen;
 	SSL_SESSION *ssl_session;
 
-	if (!_config.tls_session_cache)
+	if (!config.tls_session_cache)
 		return 0;
 
-	if (wget_tls_session_get(_config.tls_session_cache,
+	if (wget_tls_session_get(config.tls_session_cache,
 			hostname,
 			&sess, &sesslen) == 0
 		&& sess)
@@ -651,12 +1314,12 @@ static int ssl_resume_session(SSL *ssl, const char *hostname)
 		debug_printf("Found cached session data for host '%s'\n",hostname);
 		ssl_session = d2i_SSL_SESSION(NULL,
 				(const unsigned char **) &sess,
-				sesslen);
+				(long) sesslen);
 		if (!ssl_session) {
 			error_printf(_("OpenSSL: Could not parse cached session data.\n"));
 			return -1;
 		}
-#if OPENSSL_VERSION_NUMBER >= 0x10101000
+#if OPENSSL_VERSION_NUMBER >= 0x10101000 && !defined LIBRESSL_VERSION_NUMBER
 		if (!SSL_SESSION_is_resumable(ssl_session))
 			return -1;
 #endif
@@ -678,12 +1341,12 @@ static int ssl_save_session(const SSL *ssl, const char *hostname)
 	unsigned long sesslen;
 	SSL_SESSION *ssl_session = SSL_get0_session(ssl);
 
-	if (!ssl_session || !_config.tls_session_cache)
+	if (!ssl_session || !config.tls_session_cache)
 		return 0;
 
 	sesslen = i2d_SSL_SESSION(ssl_session, (unsigned char **) &sess);
 	if (sesslen) {
-		wget_tls_session_db_add(_config.tls_session_cache,
+		wget_tls_session_db_add(config.tls_session_cache,
 			wget_tls_session_new(hostname,
 				18 * 3600, /* session valid for 18 hours */
 				sess, sesslen));
@@ -706,6 +1369,95 @@ static int wait_2_read_and_write(int sockfd, int timeout)
 	return retval;
 }
 
+static bool ssl_set_alpn_offering(SSL *ssl, const char *alpn)
+{
+	int ret = WGET_E_UNKNOWN;
+	const char *s, *e;
+	char sbuf[32];
+	wget_buffer buf;
+
+	wget_buffer_init(&buf, sbuf, sizeof(sbuf));
+
+	for (s = e = alpn; *e; s = e + 1) {
+		if ((e = strchrnul(s, ',')) != s) {
+			if (e - s > 64) { // let's be reasonable
+				debug_printf("ALPN protocol too long %.*s\n", (int) (e - s), s);
+				continue;
+			}
+
+			debug_printf("ALPN offering %.*s\n", (int) (e - s), s);
+			wget_buffer_memset_append(&buf, (e - s) & 0x7F, 1); // length of protocol string
+			wget_buffer_memcat(&buf, s, e - s);
+		}
+	}
+
+	if (buf.length) {
+		if (SSL_set_alpn_protos(ssl, (unsigned char *) buf.data, (unsigned) buf.length)) {
+			debug_printf("OpenSSL: ALPN: Could not set ALPN offering");
+		} else
+			ret = WGET_E_SUCCESS;
+	}
+
+	wget_buffer_deinit(&buf);
+
+	return ret;
+}
+
+static void ssl_set_alpn_selected_protocol(const SSL *ssl, wget_tcp *tcp, wget_tls_stats_data *stats)
+{
+	const unsigned char *data;
+	unsigned int datalen;
+
+	SSL_get0_alpn_selected(ssl, &data, &datalen);
+
+	if (data && datalen) {
+		debug_printf("ALPN: Server accepted protocol '%.*s'\n", (int) datalen, data);
+
+		/* Success - Set selected protocol and update stats */
+		if (stats)
+			stats->alpn_protocol = wget_strmemdup(data, datalen);
+
+		if (datalen == 2 && data[0] == 'h' && data[1] == '2') {
+			tcp->protocol = WGET_PROTOCOL_HTTP_2_0;
+			if (stats)
+				stats->http_protocol = WGET_PROTOCOL_HTTP_2_0;
+		}
+	}
+}
+
+static int get_tls_version(const SSL *ssl)
+{
+	int version = SSL_version(ssl);
+
+	/*
+	 * These values are mapped to the return values of GnuTLS' function
+	 * gnutls_protocol_get_version() - integers on a gnutls_protocol_t enum.
+	 *
+	 * See: https://gitlab.com/gnutls/gnutls/blob/master/lib/includes/gnutls/gnutls.h.in#L736
+	 */
+	switch (version) {
+	case SSL3_VERSION:
+		/* SSL v3 */
+		return 1;
+	case TLS1_VERSION:
+		/* TLS 1.0 */
+		return 2;
+	case TLS1_1_VERSION:
+		/* TLS 1.1 */
+		return 3;
+	case TLS1_2_VERSION:
+		/* TLS 1.2 */
+		return 4;
+#if defined TLS1_3_VERSION && TLS1_2_VERSION != TLS1_3_VERSION
+	case TLS1_3_VERSION:
+		/* TLS 1.3 */
+		return 5;
+#endif
+	default:
+		return -1;
+	}
+}
+
 /**
  * \param[in] tcp A TCP connection (see wget_tcp_init())
  * \return `WGET_E_SUCCESS` on success or an error code (`WGET_E_*`) on failure
@@ -723,48 +1475,103 @@ static int wait_2_read_and_write(int sockfd, int timeout)
 int wget_ssl_open(wget_tcp *tcp)
 {
 	SSL *ssl = NULL;
+	X509_STORE *store;
 	int retval, error, resumed;
+	struct verification_flags *vflags = NULL;
+	wget_tls_stats_data stats = {
+		.alpn_protocol = NULL,
+		.version = -1,
+		.false_start = 0,
+		.tfo = 0,
+		.resumed = 0,
+		.http_protocol = WGET_PROTOCOL_HTTP_1_1,
+		.cert_chain_size = 0
+	}, *stats_p = NULL;
 
 	if (!tcp || tcp->sockfd < 0)
 		return WGET_E_INVALID;
-	if (!_init)
+	if (!init)
 		wget_ssl_init();
 
 	/* Initiate a new TLS connection from an existing OpenSSL context */
-	if (!(ssl = SSL_new(_ctx)) || !SSL_set_fd(ssl, tcp->sockfd)) {
+	if (!(ssl = SSL_new(_ctx)) || !SSL_set_fd(ssl, FD_TO_SOCKET(tcp->sockfd))) {
 		retval = WGET_E_UNKNOWN;
 		goto bail;
 	}
 
-	/* Store the hostname for the verification callback */
-	_ex_data_idx = CRYPTO_get_ex_new_index(
-		CRYPTO_EX_INDEX_SSL,
-		0, NULL,	/* argl, argp */
-		NULL,		/* new_func */
-		NULL,		/* dup_func */
-		NULL);		/* free_func */
+	/* Store state flags for the verification callback */
+	vflags = wget_malloc(sizeof(struct verification_flags));
+	if (!vflags) {
+		retval = WGET_E_MEMORY;
+		goto bail;
+	}
 
-	if (_ex_data_idx == -1
-		|| !CRYPTO_new_ex_data(CRYPTO_EX_INDEX_SSL, NULL, &_crypto_ex_data)
-		||	!CRYPTO_set_ex_data(&_crypto_ex_data, _ex_data_idx, (void *) tcp->ssl_hostname))
-	{
+	vflags->ocsp_checked = 0;
+	vflags->verifying_ocsp = 0;
+	vflags->cert_chain_size = 0;
+	vflags->hostname = tcp->ssl_hostname;
+
+	if (store_userdata_idx == -1) {
 		retval = WGET_E_UNKNOWN;
 		goto bail;
 	}
 
+	store = SSL_CTX_get_cert_store(_ctx);
+	if (!store) {
+		retval = WGET_E_UNKNOWN;
+		goto bail;
+	}
+
+	vflags->certstore = store;
+	vflags->ssl = ssl;
+
+	if (!X509_STORE_set_ex_data(store, store_userdata_idx, (void *) vflags)) {
+		retval = WGET_E_UNKNOWN;
+		goto bail;
+	}
+#ifdef WITH_OCSP
+	SSL_CTX_set_tlsext_status_arg(_ctx, vflags);
+#endif
+
+
+	/* Enable stats logging, if requested */
+	if (tls_stats_callback)
+		stats_p = &stats;
+
 	/* Enable host name verification, if requested */
-	if (_config.check_hostname) {
+	if (config.check_hostname) {
 		SSL_set1_host(ssl, tcp->ssl_hostname);
+#ifndef LIBRESSL_VERSION_NUMBER
+// LibreSSL <= 3.0.2 does not know SSL_set_hostflags()
 		SSL_set_hostflags(ssl, X509_CHECK_FLAG_NO_PARTIAL_WILDCARDS);
-	} else {
+#endif
+	}
+#if !defined LIBRESSL_VERSION_NUMBER || !defined X509_CHECK_FLAG_NEVER_CHECK_SUBJECT
+// LibreSSL <= 3.0.2 does not know SSL_set_hostflags() nor X509_CHECK_FLAG_NEVER_CHECK_SUBJECT
+// OpenSSL < 1.1 doesn't have X509_CHECK_FLAG_NEVER_CHECK_SUBJECT
+	else {
 		SSL_set_hostflags(ssl, X509_CHECK_FLAG_NEVER_CHECK_SUBJECT);
 		info_printf(_("Host name check disabled. Server certificate's subject name will not be checked.\n"));
 	}
+#endif
+
+#ifdef WITH_OCSP
+	if (config.ocsp_stapling) {
+		if (SSL_set_tlsext_status_type(ssl, TLSEXT_STATUSTYPE_ocsp))
+			debug_printf("Sending 'status_request' extension in handshake\n");
+		else
+			error_printf(_("Could not set 'status_request' extension\n"));
+	}
+#endif
 
 	/* Send Server Name Indication (SNI) */
 	if (tcp->ssl_hostname && !SSL_set_tlsext_host_name(ssl, tcp->ssl_hostname))
 		error_printf(_("SNI could not be sent"));
 
+	/* Send ALPN if requested */
+	if (config.alpn && ssl_set_alpn_offering(ssl, config.alpn))
+		error_printf(_("ALPN offering could not be sent"));
+
 	/* Resume from a previous SSL/TLS session, if available */
 	if ((resumed = ssl_resume_session(ssl, tcp->ssl_hostname)) == 1)
 		debug_printf("Will try to resume cached TLS session");
@@ -782,6 +1589,7 @@ int wget_ssl_open(wget_tcp *tcp)
 		/* Run TLS handshake */
 		retval = SSL_connect(ssl);
 		if (retval > 0) {
+			error = 0;
 			resumed = SSL_session_reused(ssl);
 			break;
 		}
@@ -812,10 +1620,32 @@ int wget_ssl_open(wget_tcp *tcp)
 	else
 		debug_printf("TLS session discarded");
 
+	/* Set the protocol selected by the server via ALPN, if any */
+	if (config.alpn)
+		ssl_set_alpn_selected_protocol(ssl, tcp, stats_p);
+
+	if (stats_p) {
+		stats_p->version = get_tls_version(ssl);
+		stats_p->hostname = vflags->hostname;
+		stats_p->resumed = resumed;
+		stats_p->cert_chain_size = vflags->cert_chain_size;
+		tls_stats_callback(stats_p, tls_stats_ctx);
+		xfree(stats_p->alpn_protocol);
+
+#ifdef MSG_FASTOPEN
+		stats_p->tfo = wget_tcp_get_tcp_fastopen(tcp);
+#endif
+	}
+
+	tcp->hpkp = vflags->hpkp_stats;
 	tcp->ssl_session = ssl;
+	xfree(vflags);
 	return WGET_E_SUCCESS;
 
 bail:
+	if (stats_p)
+		xfree(stats_p->alpn_protocol);
+	xfree(vflags);
 	if (ssl)
 		SSL_free(ssl);
 	return retval;
@@ -841,9 +1671,6 @@ void wget_ssl_close(void **session)
 			retval = SSL_shutdown(ssl);
 		while (retval == 0);
 
-		CRYPTO_free_ex_data(_ex_data_idx, NULL, &_crypto_ex_data);
-		CRYPTO_free_ex_index(CRYPTO_EX_INDEX_SSL, _ex_data_idx);
-
 		SSL_free(ssl);
 		*session = NULL;
 	}
@@ -860,13 +1687,9 @@ static int ssl_transfer(int want,
 		return 0;
 	if ((ssl = session) == NULL)
 		return WGET_E_INVALID;
-	if ((fd = SSL_get_fd(ssl)) < 0)
+	if ((fd = SOCKET_TO_FD(SSL_get_fd(ssl))) < 0)
 		return WGET_E_UNKNOWN;
 
-	/* SSL_read() and SSL_write() take ints, so we'd rather play safe here */
-	if (count > INT_MAX)
-		count = INT_MAX;
-
 	if (timeout < -1)
 		timeout = -1;
 
