@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016-2021 Free Software Foundation, Inc.
+ * Copyright (c) 2016-2022 Free Software Foundation, Inc.
  *
  * This file is part of libwget.
  *
@@ -361,8 +361,11 @@ int wget_tls_session_db_load(wget_tls_session_db *tls_session_db, const char *fn
 }
 
 WGET_GCC_NONNULL_ALL
-static int tls_session_save(FILE *fp, const wget_tls_session *tls_session)
+static int tls_session_save(void *_fp, const void *_tls_session, WGET_GCC_UNUSED void *v)
 {
+	FILE *fp = _fp;
+	const wget_tls_session *tls_session = _tls_session;
+
 	char session_b64[wget_base64_get_encoded_length(tls_session->data_size)];
 
 	wget_base64_encode(session_b64, (const char *) tls_session->data, tls_session->data_size);
@@ -380,7 +383,7 @@ static int tls_session_db_save(void *tls_session_db, FILE *fp)
 		fputs("#Generated by libwget " PACKAGE_VERSION ". Edit at your own risk.\n", fp);
 		fputs("#<hostname>  <created> <max-age> <session data>\n\n", fp);
 
-		wget_hashmap_browse(entries, (wget_hashmap_browse_fn *) tls_session_save, fp);
+		wget_hashmap_browse(entries, tls_session_save, fp);
 
 		if (ferror(fp))
 			return -1;
