@@ -1,21 +1,21 @@
 /*
- * Copyright(c) 2013 Tim Ruehsen
- * Copyright(c) 2015-2019 Free Software Foundation, Inc.
+ * Copyright (c) 2013 Tim Ruehsen
+ * Copyright (c) 2015-2022 Free Software Foundation, Inc.
  *
- * This file is part of libwget.
+ * This file is part of Wget
  *
- * Libwget is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
+ * Wget is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * Libwget is distributed in the hope that it will be useful,
+ * Wget is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with libwget.  If not, see <https://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with Wget  If not, see <https://www.gnu.org/licenses/>.
  *
  *
  * Testing Wget
@@ -36,18 +36,18 @@ int main(void)
 	wget_test_url_t urls[]={
 		{	.name = "/main.rss",
 			.body =
-"<?xml version=\"1.0\" encoding=\"utf-8\"?>"\
-"<channel>"\
-"<title>Da title</title>"\
-"<link>http://localhost:{{port}}/page1.html</link>"\
-"<description>Nice article</description>"\
-"<item>"\
-"<title>Item title</title>"\
-"<link>http://localhost:{{port}}/page2.html</link>"\
-"<pubDate>Sun, 01 Sep 2013 18:41:05 -0700</pubDate>"\
-"<guid isPermaLink=\"true\">http://localhost:{{port}}/page3.html</guid>"\
-"<description>item description</description>"\
-"</item></channel></rss>",
+				"<?xml version=\"1.0\" encoding=\"utf-8\"?>"\
+				"<channel>"\
+				"<title>Da title</title>"\
+				"<link>http://localhost:{{port}}/page1.html</link>"\
+				"<description>Nice article</description>"\
+				"<item>"\
+				"<title>Item title</title>"\
+				"<link>http://localhost:{{port}}/page2.html</link>"\
+				"<pubDate>Sun, 01 Sep 2013 18:41:05 -0700</pubDate>"\
+				"<guid isPermaLink=\"true\">http://localhost:{{port}}/page3.html</guid>"\
+				"<description>item description</description>"\
+				"</item></channel></rss>",
 		},
 		{	.name = "/page1.html",
 			.code = "200 Dontcare",
@@ -95,5 +95,5 @@ int main(void)
 			{	NULL } },
 		0);
 
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
