@@ -1,21 +1,21 @@
 /*
- * Copyright(c) 2013 Tim Ruehsen
- * Copyright(c) 2015-2019 Free Software Foundation, Inc.
+ * Copyright (c) 2013 Tim Ruehsen
+ * Copyright (c) 2015-2022 Free Software Foundation, Inc.
  *
- * This file is part of libwget.
+ * This file is part of Wget
  *
- * Libwget is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
+ * Wget is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
- * Libwget is distributed in the hope that it will be useful,
+ * Wget is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with libwget.  If not, see <https://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU General Public License
+ * along with Wget  If not, see <https://www.gnu.org/licenses/>.
  *
  *
  * Testing Wget
@@ -477,6 +477,32 @@ int main(void)
 			{	NULL } },
 		0);
 
+	// test -c with existing empty file
+	wget_test(
+		WGET_TEST_OPTIONS, "-c",
+		WGET_TEST_REQUEST_URL, "index.html",
+		WGET_TEST_EXISTING_FILES, &(wget_test_file_t []) {
+			{ urls[0].name + 1, "" },
+			{	NULL } },
+		WGET_TEST_EXPECTED_ERROR_CODE, 0,
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{ urls[0].name + 1, urls[0].body },
+			{	NULL } },
+		0);
+
+	// test -c with existing complete file
+	wget_test(
+		WGET_TEST_OPTIONS, "-c",
+		WGET_TEST_REQUEST_URL, "index.html",
+		WGET_TEST_EXISTING_FILES, &(wget_test_file_t []) {
+			{ urls[0].name + 1, urls[0].body },
+			{	NULL } },
+		WGET_TEST_EXPECTED_ERROR_CODE, 0,
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{ urls[0].name + 1, urls[0].body },
+			{	NULL } },
+		0);
+
 	// test --chunk-size, new file
 	wget_test(
 		WGET_TEST_OPTIONS, "--chunk-size=3",
@@ -488,6 +514,17 @@ int main(void)
 			{	NULL } },
 		0);
 
+	// test --chunk-size with --progress, new file
+	wget_test(
+		WGET_TEST_OPTIONS, "--chunk-size=3 --progress=bar",
+		WGET_TEST_REQUEST_URL, "dummy.txt",
+		WGET_TEST_EXPECTED_ERROR_CODE, 0,
+		// no existing file
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{	"dummy.txt", urls[3].body },
+			{	NULL } },
+		0);
+
 	// test --chunk-size, new file, without Content-Length header
 	wget_test(
 		WGET_TEST_OPTIONS, "--chunk-size=3",
@@ -500,6 +537,18 @@ int main(void)
 			{	NULL } },
 		0);
 
+	// test --chunk-size with --progress, new file, without Content-Length header
+	wget_test(
+		WGET_TEST_OPTIONS, "--chunk-size=3 --progress=bar",
+		WGET_TEST_REQUEST_URL, "dummy.txt",
+		WGET_TEST_SERVER_SEND_CONTENT_LENGTH, 0, // server does not send Content-Length
+		WGET_TEST_EXPECTED_ERROR_CODE, 0,
+		// no existing file
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{	"dummy.txt", urls[3].body },
+			{	NULL } },
+		0);
+
 	// test --chunk-size, new file, with chunk size > Content-Length
 	wget_test(
 		WGET_TEST_OPTIONS, "--chunk-size=1000",
@@ -511,6 +560,17 @@ int main(void)
 			{	NULL } },
 		0);
 
+	// test --chunk-size with --progress, new file, with chunk size > Content-Length
+	wget_test(
+		WGET_TEST_OPTIONS, "--chunk-size=1000 --progress=bar",
+		WGET_TEST_REQUEST_URL, "dummy.txt",
+		WGET_TEST_EXPECTED_ERROR_CODE, 0,
+		// no existing file
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{	"dummy.txt", urls[3].body },
+			{	NULL } },
+		0);
+
 	// test -c --chunk-size, new file
 	wget_test(
 		WGET_TEST_OPTIONS, "--chunk-size=3 -c",
@@ -522,6 +582,17 @@ int main(void)
 			{	NULL } },
 		0);
 
+	// test -c --chunk-size with --progress, new file
+	wget_test(
+		WGET_TEST_OPTIONS, "--chunk-size=3 -c --progress=bar",
+		WGET_TEST_REQUEST_URL, "dummy.txt",
+		WGET_TEST_EXPECTED_ERROR_CODE, 0,
+		// no existing file
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{	"dummy.txt", urls[3].body },
+			{	NULL } },
+		0);
+
 	// test -c --chunk-size, existing file
 	wget_test(
 		WGET_TEST_OPTIONS, "--chunk-size=3 -c",
@@ -535,6 +606,19 @@ int main(void)
 			{	NULL } },
 		0);
 
+	// test -c --chunk-size with --progress, existing file
+	wget_test(
+		WGET_TEST_OPTIONS, "--chunk-size=3 -c --progress=bar",
+		WGET_TEST_REQUEST_URL, "dummy.txt",
+		WGET_TEST_EXPECTED_ERROR_CODE, 0,
+		WGET_TEST_EXISTING_FILES, &(wget_test_file_t []) {
+			{ "dummy.txt",  urls[3].body },
+			{	NULL } },
+		WGET_TEST_EXPECTED_FILES, &(wget_test_file_t []) {
+			{	"dummy.txt", urls[3].body },
+			{	NULL } },
+		0);
+
 	// test--https-only
 	wget_test(
 		WGET_TEST_OPTIONS, "--https-only -r -nH",
@@ -564,5 +648,5 @@ int main(void)
 			{	NULL } },
 		0);
 
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
